@@ -1,12 +1,11 @@
 #include <filesystem>
-#include <fstream>
 #include <getopt.h>
 #include <iostream>
+#include <string>
 
 #include "aot.h"
 #include "bpftrace.h"
 #include "log.h"
-#include "output.h"
 #include "run_bpftrace.h"
 #include "version.h"
 
@@ -28,36 +27,6 @@ void usage(std::ostream& out, std::string_view filename)
   out << "    -V, --version  bpftrace version" << std::endl;
   out << std::endl;
   // clang-format on
-  return;
-}
-
-std::unique_ptr<Output> prepare_output(const std::string& output_file,
-                                       const std::string& output_format)
-{
-  std::ostream* os = &std::cout;
-  std::ofstream outputstream;
-  if (!output_file.empty()) {
-    outputstream.open(output_file);
-    if (outputstream.fail()) {
-      LOG(ERROR) << "Failed to open output file: \"" << output_file
-                 << "\": " << strerror(errno);
-      return nullptr;
-    }
-    os = &outputstream;
-  }
-
-  std::unique_ptr<Output> output;
-  if (output_format.empty() || output_format == "text") {
-    output = std::make_unique<TextOutput>(*os);
-  } else if (output_format == "json") {
-    output = std::make_unique<JsonOutput>(*os);
-  } else {
-    LOG(ERROR) << "Invalid output format \"" << output_format << "\"\n"
-               << "Valid formats: 'text', 'json'";
-    return nullptr;
-  }
-
-  return output;
 }
 
 int main(int argc, char* argv[])
@@ -65,12 +34,30 @@ int main(int argc, char* argv[])
   std::string output_file, output_format;
   int c;
 
+  std::vector<std::string> named_params;
+
   // TODO: which other options from `bpftrace` should be included?
   const char* const short_opts = "d:f:hVo:qv";
   option long_opts[] = {
-    option{ "help", no_argument, nullptr, 'h' },
-    option{ "version", no_argument, nullptr, 'V' },
-    option{ nullptr, 0, nullptr, 0 }, // Must be last
+    option{
+        .name = "help",
+        .has_arg = no_argument,
+        .flag = nullptr,
+        .val = 'h',
+    },
+    option{
+        .name = "version",
+        .has_arg = no_argument,
+        .flag = nullptr,
+        .val = 'V',
+    },
+    // Must be last
+    option{
+        .name = nullptr,
+        .has_arg = 0,
+        .flag = nullptr,
+        .val = 0,
+    },
   };
 
   std::filesystem::path p(argv[0]);
@@ -101,11 +88,11 @@ int main(int argc, char* argv[])
         bt_verbose = true;
         break;
       case 'd':
-        if (std::strcmp(optarg, "libbpf") == 0)
+        if (std::string(optarg) == "libbpf")
           bt_debug.insert(DebugStage::Libbpf);
-        else if (std::strcmp(optarg, "verifier") == 0)
+        else if (std::string(optarg) == "verifier")
           bt_debug.insert(DebugStage::Verifier);
-        else if (std::strcmp(optarg, "all") == 0) {
+        else if (std::string(optarg) == "all") {
           bt_debug.insert({ DebugStage::Libbpf, DebugStage::Verifier });
         } else {
           LOG(ERROR) << "USAGE: invalid option for -d: " << optarg;
@@ -118,6 +105,18 @@ int main(int argc, char* argv[])
     }
   }
 
+  while (optind < argc) {
+    auto pos_arg = std::string(argv[optind]);
+    if (pos_arg.starts_with("--")) {
+      named_params.emplace_back(pos_arg.substr(2));
+    } else {
+      // AOT does not support positional parameters
+      LOG(ERROR) << "AOT does not support positional parameters";
+      return 1;
+    }
+    optind++;
+  }
+
   if (argv[optind]) {
     usage(std::cerr, argv[0]);
     return 1;
@@ -127,11 +126,7 @@ int main(int argc, char* argv[])
 
   libbpf_set_print(libbpf_print);
 
-  auto output = prepare_output(output_file, output_format);
-  if (!output)
-    return 1;
-
-  BPFtrace bpftrace(std::move(output));
+  BPFtrace bpftrace;
 
   int err = aot::load(bpftrace, argv[0]);
   if (err) {
@@ -139,5 +134,14 @@ int main(int argc, char* argv[])
     return err;
   }
 
-  return run_bpftrace(bpftrace, bpftrace.bytecode_);
+  // FIXME(#4087): We should serialize the C enum definitions as part of the AOT
+  // payload in order to allow this printing to work.
+  ast::CDefinitions no_c_defs;
+
+  return run_bpftrace(bpftrace,
+                      output_file,
+                      output_format,
+                      no_c_defs,
+                      bpftrace.bytecode_,
+                      std::move(named_params));
 }
