@@ -1,18 +1,22 @@
-#include "ast/attachpoint_parser.h"
-
-#include "ast.h"
-#include "ast/int_parser.h"
-#include "log.h"
-#include "types.h"
 #include <algorithm>
 #include <bcc/bcc_proc.h>
+#include <cctype>
 #include <exception>
-#include <functional>
 #include <iostream>
 #include <string>
-#include <unordered_map>
 #include <vector>
 
+#include "ast/ast.h"
+#include "ast/attachpoint_parser.h"
+#include "ast/context.h"
+#include "ast/helpers.h"
+#include "types.h"
+#include "util/int_parser.h"
+#include "util/paths.h"
+#include "util/strings.h"
+#include "util/system.h"
+#include "util/wildcard.h"
+
 namespace bpftrace::ast {
 
 AttachPointParser::State AttachPointParser::argument_count_error(
@@ -31,31 +35,10 @@ AttachPointParser::State AttachPointParser::argument_count_error(
   return INVALID;
 }
 
-std::optional<uint64_t> AttachPointParser::stoull(const std::string &str)
-{
-  try {
-    return int_parser::to_uint(str, 0);
-  } catch (const std::exception &e) {
-    errs_ << e.what() << std::endl;
-    return std::nullopt;
-  }
-}
-
-std::optional<int64_t> AttachPointParser::stoll(const std::string &str)
-{
-  try {
-    return int_parser::to_int(str, 0);
-  } catch (const std::exception &e) {
-    errs_ << e.what() << std::endl;
-    return std::nullopt;
-  }
-}
-
 AttachPointParser::AttachPointParser(ASTContext &ctx,
                                      BPFtrace &bpftrace,
-                                     std::ostream &sink,
                                      bool listing)
-    : ctx_(ctx), bpftrace_(bpftrace), sink_(sink), listing_(listing)
+    : ctx_(ctx), bpftrace_(bpftrace), listing_(listing)
 {
 }
 
@@ -67,14 +50,14 @@ int AttachPointParser::parse()
   uint32_t failed = 0;
   for (Probe *probe : ctx_.root->probes) {
     for (size_t i = 0; i < probe->attach_points.size(); ++i) {
-      auto ap_ptr = probe->attach_points[i];
+      auto *ap_ptr = probe->attach_points[i];
       auto &ap = *ap_ptr;
       new_attach_points.clear();
 
       State s = parse_attachpoint(ap);
       if (s == INVALID) {
         ++failed;
-        LOG(ERROR, ap.loc, sink_) << errs_.str();
+        ap.addError() << errs_.str();
       } else if (s == SKIP || s == NEW_APS) {
         // Remove the current attach point
         probe->attach_points.erase(probe->attach_points.begin() + i);
@@ -92,15 +75,14 @@ int AttachPointParser::parse()
       errs_.str({});
     }
 
-    auto new_end = std::remove_if(probe->attach_points.begin(),
-                                  probe->attach_points.end(),
-                                  [](const AttachPoint *ap) {
-                                    return ap->provider.empty();
-                                  });
-    probe->attach_points.erase(new_end, probe->attach_points.end());
+    auto it = std::ranges::remove_if(probe->attach_points,
+                                     [](const AttachPoint *ap) {
+                                       return ap->provider.empty();
+                                     });
+    probe->attach_points.erase(it.begin(), it.end());
 
     if (probe->attach_points.empty()) {
-      LOG(ERROR, probe->loc, sink_) << "No attach points for probe";
+      probe->addError() << "No attach points for probe";
       failed++;
     }
   }
@@ -128,7 +110,7 @@ AttachPointParser::State AttachPointParser::parse_attachpoint(AttachPoint &ap)
   }
 
   std::set<std::string> probe_types;
-  if (has_wildcard(parts_.front())) {
+  if (util::has_wildcard(parts_.front())) {
     // Single argument listing looks at all relevant probe types
     std::string probetype_query = (parts_.size() == 1) ? "*" : parts_.front();
 
@@ -136,7 +118,7 @@ AttachPointParser::State AttachPointParser::parse_attachpoint(AttachPoint &ap)
     // If PID is specified or the second part of the attach point is a path
     // (contains '/'), use userspace probe types.
     // Otherwise, use kernel probe types.
-    if (bpftrace_.pid() > 0 ||
+    if (bpftrace_.pid().has_value() ||
         (parts_.size() >= 2 && parts_[1].find('/') != std::string::npos)) {
       probe_types = bpftrace_.probe_matcher_->expand_probetype_userspace(
           probetype_query);
@@ -148,7 +130,7 @@ AttachPointParser::State AttachPointParser::parse_attachpoint(AttachPoint &ap)
     probe_types = { parts_.front() };
 
   if (probe_types.empty()) {
-    if (has_wildcard(parts_.front()))
+    if (util::has_wildcard(parts_.front()))
       errs_ << "No probe type matched for " << parts_.front() << std::endl;
     else
       errs_ << "Invalid probe type: " << parts_.front() << std::endl;
@@ -159,13 +141,13 @@ AttachPointParser::State AttachPointParser::parse_attachpoint(AttachPoint &ap)
     for (const auto &probe_type : probe_types) {
       std::string raw_input = ap.raw_input;
       if (parts_.size() > 1)
-        erase_prefix(raw_input);
+        util::erase_prefix(raw_input);
       raw_input = probe_type + ":" + raw_input;
       // New attach points have ignore_invalid set to true - probe types for
       // which raw_input has invalid number of parts will be ignored (instead
       // of throwing an error). These will have the same associated location.
       new_attach_points.push_back(
-          ctx_.make_node<AttachPoint>(raw_input, true, ap.loc));
+          ctx_.make_node<AttachPoint>(raw_input, true, Location(ap.loc)));
     }
     return NEW_APS;
   }
@@ -175,6 +157,8 @@ AttachPointParser::State AttachPointParser::parse_attachpoint(AttachPoint &ap)
   switch (probetype(ap.provider)) {
     case ProbeType::special:
       return special_parser();
+    case ProbeType::benchmark:
+      return benchmark_parser();
     case ProbeType::kprobe:
       return kprobe_parser();
     case ProbeType::kretprobe:
@@ -235,31 +219,39 @@ AttachPointParser::State AttachPointParser::lex_attachpoint(
       argument += raw[idx + 1];
       ++idx;
     } else if (!in_quotes && raw[idx] == '$') {
-      // There's an assumption that the positional parameter is well
-      // formed. ie we are not expecting a bare `$` or `$nonint`. The
-      // bison parser should have guaranteed this.
       size_t i = idx + 1;
       size_t len = 0;
-      while (i < raw.size() && (raw[i] != '"' && raw[i] != ':')) {
+      while (i < raw.size() && std::isdigit(raw[i])) {
+        if (len == 0 && raw[i] == '0') {
+          break;
+        }
         len++;
         i++;
       }
 
-      std::string param_idx_str = raw.substr(idx + 1, len);
-      size_t pos, param_idx;
-      param_idx = std::stoll(param_idx_str, &pos, 0);
+      std::string param_idx_str;
 
-      if (pos != param_idx_str.size()) {
+      if (len == 0 && (idx + 1) < raw.size()) {
+        param_idx_str = raw.substr(idx + 1, 1);
         errs_
-            << "Found trailing text '" << param_idx_str.substr(pos)
-            << "' in positional parameter index. Try quoting the trailing text."
-            << std::endl;
+            << "invalid trailing character for positional param: "
+            << param_idx_str
+            << ". Try quoting this entire part if this is intentional e.g. \"$"
+            << param_idx_str << "\".";
+        return State::INVALID;
+      }
+
+      param_idx_str = raw.substr(idx + 1, len);
+      auto param_idx = util::to_uint(param_idx_str, 10);
+      if (!param_idx) {
+        errs_ << "positional parameter is not valid: " << param_idx.takeError()
+              << std::endl;
         return State::INVALID;
       }
 
       // Expand the positional param in-place and decrement idx so that the next
       // iteration takes the first char of the expansion
-      raw = raw.substr(0, idx) + bpftrace_.get_param(param_idx, true) +
+      raw = raw.substr(0, idx) + bpftrace_.get_param(*param_idx) +
             raw.substr(i);
       idx--;
     } else
@@ -277,11 +269,11 @@ AttachPointParser::State AttachPointParser::lex_attachpoint(
 
 AttachPointParser::State AttachPointParser::special_parser()
 {
-  // Can only have reached here if provider is `BEGIN` or `END` or `self`
-  assert(ap_->provider == "BEGIN" || ap_->provider == "END" ||
+  // Can only have reached here if provider is `begin` or `end` or `self`
+  assert(ap_->provider == "begin" || ap_->provider == "end" ||
          ap_->provider == "self");
 
-  if (ap_->provider == "BEGIN" || ap_->provider == "END") {
+  if (ap_->provider == "begin" || ap_->provider == "end") {
     if (parts_.size() == 2 && parts_[1] == "*")
       parts_.pop_back();
     if (parts_.size() != 1) {
@@ -298,6 +290,22 @@ AttachPointParser::State AttachPointParser::special_parser()
   return OK;
 }
 
+AttachPointParser::State AttachPointParser::benchmark_parser()
+{
+  // Can only have reached here if provider is `bench`
+  assert(ap_->provider == "bench");
+
+  if (ap_->provider == "bench") {
+    if (parts_.size() != 2) {
+      return argument_count_error(1);
+    }
+
+    ap_->target = parts_[1];
+  }
+
+  return OK;
+}
+
 AttachPointParser::State AttachPointParser::kprobe_parser(bool allow_offset)
 {
   auto num_parts = parts_.size();
@@ -329,7 +337,7 @@ AttachPointParser::State AttachPointParser::kprobe_parser(bool allow_offset)
       return INVALID;
     }
 
-    auto offset_parts = split_string(parts_[func_idx], '+', true);
+    auto offset_parts = util::split_string(parts_[func_idx], '+', true);
     if (offset_parts.size() != 2) {
       errs_ << "Invalid offset" << std::endl;
       return INVALID;
@@ -337,9 +345,9 @@ AttachPointParser::State AttachPointParser::kprobe_parser(bool allow_offset)
 
     ap_->func = offset_parts[0];
 
-    auto res = stoll(offset_parts[1]);
+    auto res = util::to_uint(offset_parts[1]);
     if (!res) {
-      errs_ << "Invalid offset" << std::endl;
+      errs_ << "Invalid offset: " << res.takeError() << std::endl;
       return INVALID;
     }
     ap_->func_offset = *res;
@@ -349,18 +357,6 @@ AttachPointParser::State AttachPointParser::kprobe_parser(bool allow_offset)
     ap_->func = parts_[func_idx];
   }
 
-  // kprobe_multi does not support the "module:function" syntax so in case
-  // a module is specified, always use full expansion
-  if (has_wildcard(ap_->target))
-    ap_->expansion = ExpansionType::FULL;
-  else if (has_wildcard(ap_->func)) {
-    if (ap_->target.empty() && bpftrace_.feature_->has_kprobe_multi()) {
-      ap_->expansion = ExpansionType::MULTI;
-    } else {
-      ap_->expansion = ExpansionType::FULL;
-    }
-  }
-
   return OK;
 }
 
@@ -372,15 +368,16 @@ AttachPointParser::State AttachPointParser::kretprobe_parser()
 AttachPointParser::State AttachPointParser::uprobe_parser(bool allow_offset,
                                                           bool allow_abs_addr)
 {
-  if (bpftrace_.pid() > 0 &&
+  const auto pid = bpftrace_.pid();
+  if (pid.has_value() &&
       (parts_.size() == 2 ||
        (parts_.size() == 3 && is_supported_lang(parts_[1])))) {
     // For PID, the target may be skipped
     if (parts_.size() == 2)
       parts_.insert(parts_.begin() + 1, "");
 
-    auto target = get_pid_exe(bpftrace_.pid());
-    parts_[1] = path_for_pid_mountns(bpftrace_.pid(), target);
+    auto target = util::get_pid_exe(*pid);
+    parts_[1] = target ? util::path_for_pid_mountns(*pid, *target) : "";
   }
 
   if (parts_.size() != 3 && parts_.size() != 4) {
@@ -395,12 +392,13 @@ AttachPointParser::State AttachPointParser::uprobe_parser(bool allow_offset,
 
   ap_->target = "";
 
-  if (!has_wildcard(parts_[1]) && parts_[1].find("lib") == 0) {
+  if (!util::has_wildcard(parts_[1]) && parts_[1].starts_with("lib")) {
     // Automatic resolution of shared library paths.
     // If the target has form "libXXX" then we use BCC to find the correct path
     // to the given library as it may differ across systems.
     auto libname = parts_[1].substr(3);
-    auto lib_path = bcc_procutils_which_so(libname.c_str(), bpftrace_.pid());
+    auto *lib_path = bcc_procutils_which_so(libname.c_str(),
+                                            bpftrace_.pid().value_or(0));
     if (lib_path) {
       ap_->target = lib_path;
       ::free(lib_path);
@@ -425,17 +423,16 @@ AttachPointParser::State AttachPointParser::uprobe_parser(bool allow_offset,
       return INVALID;
     }
 
-    auto offset_parts = split_string(func, '+', true);
+    auto offset_parts = util::split_string(func, '+', true);
     if (offset_parts.size() != 2) {
       errs_ << "Invalid offset" << std::endl;
       return INVALID;
     }
 
     ap_->func = offset_parts[0];
-
-    auto res = stoll(offset_parts[1]);
+    auto res = util::to_uint(offset_parts[1]);
     if (!res) {
-      errs_ << "Invalid offset" << std::endl;
+      errs_ << "Invalid offset: " << res.takeError() << std::endl;
       return INVALID;
     }
     ap_->func_offset = *res;
@@ -443,9 +440,9 @@ AttachPointParser::State AttachPointParser::uprobe_parser(bool allow_offset,
   // Default case (eg uprobe:[addr][func])
   else {
     if (allow_abs_addr) {
-      auto res = stoll(func);
+      auto res = util::to_uint(func);
       if (res) {
-        if (has_wildcard(ap_->target)) {
+        if (util::has_wildcard(ap_->target)) {
           errs_ << "Cannot use wildcards with absolute address" << std::endl;
           return INVALID;
         }
@@ -458,18 +455,6 @@ AttachPointParser::State AttachPointParser::uprobe_parser(bool allow_offset,
       ap_->func = func;
   }
 
-  // As the C++ language supports function overload, a given function name
-  // (without parameters) could have multiple matches even when no
-  // wildcards are used.
-  if (has_wildcard(ap_->func) || has_wildcard(ap_->target) ||
-      ap_->lang == "cpp") {
-    if (bpftrace_.feature_->has_uprobe_multi()) {
-      ap_->expansion = ExpansionType::MULTI;
-    } else {
-      ap_->expansion = ExpansionType::FULL;
-    }
-  }
-
   return OK;
 }
 
@@ -480,7 +465,7 @@ AttachPointParser::State AttachPointParser::uretprobe_parser()
 
 AttachPointParser::State AttachPointParser::usdt_parser()
 {
-  if (bpftrace_.pid() > 0) {
+  if (bpftrace_.pid().has_value()) {
     // For PID, the target can be skipped
     if (parts_.size() == 2) {
       parts_.push_back(parts_[1]);
@@ -503,11 +488,6 @@ AttachPointParser::State AttachPointParser::usdt_parser()
     ap_->func = parts_[3];
   }
 
-  // Always fully expand USDT probes as they may access args
-  if (has_wildcard(ap_->target) || has_wildcard(ap_->ns) || ap_->ns.empty() ||
-      has_wildcard(ap_->func) || bpftrace_.pid())
-    ap_->expansion = ExpansionType::FULL;
-
   return OK;
 }
 
@@ -516,7 +496,7 @@ AttachPointParser::State AttachPointParser::tracepoint_parser()
   // Help with `bpftrace -l 'tracepoint:*foo*'` listing -- wildcard the
   // tracepoint category b/c user is most likely to be looking for the event
   // name
-  if (parts_.size() == 2 && has_wildcard(parts_.at(1)))
+  if (parts_.size() == 2 && util::has_wildcard(parts_.at(1)))
     parts_.insert(parts_.begin() + 1, "*");
 
   if (parts_.size() != 3) {
@@ -529,60 +509,62 @@ AttachPointParser::State AttachPointParser::tracepoint_parser()
   ap_->target = parts_[1];
   ap_->func = parts_[2];
 
-  if (ap_->target.find('*') != std::string::npos ||
-      ap_->func.find('*') != std::string::npos)
-    ap_->expansion = ExpansionType::FULL;
+  return OK;
+}
 
+// Used for both profile and interval probes
+AttachPointParser::State AttachPointParser::frequency_parser()
+{
+  if (parts_.size() == 2) {
+    if (util::has_wildcard(parts_[1])) {
+      // Wildcards are allowed for listing
+      ap_->target = parts_[1];
+      ap_->freq = 0;
+      return OK;
+    }
+
+    auto res = util::to_uint(parts_[1]);
+    if (!res) {
+      errs_ << "Invalid rate of " << ap_->provider
+            << " probe: " << res.takeError() << std::endl;
+      return INVALID;
+    }
+    if (*res < 1000) {
+      errs_ << "Invalid rate of " << ap_->provider
+            << " probe. Minimum is 1000 or 1us. Found: " << *res
+            << " nanoseconds" << std::endl;
+      return INVALID;
+    }
+    ap_->target = "us";
+    // res is in nanoseconds
+    ap_->freq = (*res / 1000);
+    return OK;
+  }
+
+  if (parts_.size() != 3) {
+    return argument_count_error(1, 2);
+  }
+
+  ap_->target = parts_[1];
+  auto res = util::to_uint(parts_[2]);
+  if (!res) {
+    errs_ << "Invalid rate of " << ap_->provider
+          << " probe: " << res.takeError() << std::endl;
+    return INVALID;
+  }
+
+  ap_->freq = *res;
   return OK;
 }
 
 AttachPointParser::State AttachPointParser::profile_parser()
 {
-  if (parts_.size() == 2 && has_wildcard(parts_[1])) {
-    // Wildcards are allowed for listing
-    ap_->target = parts_[1];
-    ap_->freq = 0;
-    return OK;
-  }
-
-  if (parts_.size() != 3) {
-    return argument_count_error(2);
-  }
-
-  ap_->target = parts_[1];
-
-  auto res = stoull(parts_[2]);
-  if (!res) {
-    errs_ << "Invalid rate of " << ap_->provider << " probe";
-    return INVALID;
-  }
-
-  ap_->freq = *res;
-  return OK;
+  return frequency_parser();
 }
 
 AttachPointParser::State AttachPointParser::interval_parser()
 {
-  if (parts_.size() == 2 && has_wildcard(parts_[1])) {
-    // Wildcards are allowed for listing
-    ap_->target = parts_[1];
-    ap_->freq = 0;
-    return OK;
-  }
-
-  if (parts_.size() != 3) {
-    return argument_count_error(2);
-  }
-
-  ap_->target = parts_[1];
-  auto res = stoull(parts_[2]);
-  if (!res) {
-    errs_ << "Invalid rate of " << ap_->provider << " probe";
-    return INVALID;
-  }
-
-  ap_->freq = *res;
-  return OK;
+  return frequency_parser();
 }
 
 AttachPointParser::State AttachPointParser::software_parser()
@@ -597,9 +579,10 @@ AttachPointParser::State AttachPointParser::software_parser()
   ap_->target = parts_[1];
 
   if (parts_.size() == 3 && parts_[2] != "*") {
-    auto res = stoull(parts_[2]);
+    auto res = util::to_uint(parts_[2]);
     if (!res) {
-      errs_ << "Invalid count for " << ap_->provider << " probe";
+      errs_ << "Invalid count for " << ap_->provider
+            << " probe: " << res.takeError() << std::endl;
       return INVALID;
     }
     ap_->freq = *res;
@@ -620,9 +603,10 @@ AttachPointParser::State AttachPointParser::hardware_parser()
   ap_->target = parts_[1];
 
   if (parts_.size() == 3 && parts_[2] != "*") {
-    auto res = stoull(parts_[2]);
+    auto res = util::to_uint(parts_[2]);
     if (!res) {
-      errs_ << "Invalid count for " << ap_->provider << " probe";
+      errs_ << "Invalid count for " << ap_->provider
+            << " probe: " << res.takeError() << std::endl;
       return INVALID;
     }
     ap_->freq = *res;
@@ -638,39 +622,40 @@ AttachPointParser::State AttachPointParser::watchpoint_parser(bool async)
   }
 
   if (parts_[1].find('+') == std::string::npos) {
-    auto parsed = stoull(parts_[1]);
+    auto parsed = util::to_uint(parts_[1]);
     if (!parsed) {
-      errs_ << "Invalid function/address argument" << std::endl;
+      errs_ << "Invalid function/address argument: " << parsed.takeError()
+            << std::endl;
       return INVALID;
     }
     ap_->address = *parsed;
   } else {
-    auto func_arg_parts = split_string(parts_[1], '+', true);
+    auto func_arg_parts = util::split_string(parts_[1], '+', true);
     if (func_arg_parts.size() != 2) {
-      errs_ << "Invalid function/address argument" << std::endl;
+      errs_ << "Invalid function/address argument: " << parts_[1] << std::endl;
       return INVALID;
     }
 
     ap_->func = func_arg_parts[0];
-    if (ap_->func.find('*') != std::string::npos)
-      ap_->expansion = ExpansionType::FULL;
 
-    if (func_arg_parts[1].size() <= 3 || func_arg_parts[1].find("arg") != 0) {
-      errs_ << "Invalid function argument" << std::endl;
+    if (func_arg_parts[1].size() <= 3 ||
+        !func_arg_parts[1].starts_with("arg")) {
+      errs_ << "Invalid function/address argument: " << func_arg_parts[1]
+            << std::endl;
       return INVALID;
     }
 
-    auto parsed = stoull(func_arg_parts[1].substr(3));
+    auto parsed = util::to_uint(func_arg_parts[1].substr(3));
     if (!parsed) {
-      errs_ << "Invalid function argument" << std::endl;
+      errs_ << "Invalid function argument: " << parsed.takeError() << std::endl;
       return INVALID;
     }
     ap_->address = *parsed;
   }
 
-  auto len_parsed = stoull(parts_[2]);
+  auto len_parsed = util::to_uint(parts_[2]);
   if (!len_parsed) {
-    errs_ << "Invalid length argument" << std::endl;
+    errs_ << "Invalid length argument: " << len_parsed.takeError() << std::endl;
     return INVALID;
   }
   ap_->len = *len_parsed;
@@ -688,11 +673,39 @@ AttachPointParser::State AttachPointParser::watchpoint_parser(bool async)
 AttachPointParser::State AttachPointParser::fentry_parser()
 {
   // fentry[:module]:function
-  if (parts_.size() != 2 && parts_.size() != 3) {
+  // fentry:bpf:[:prog_id]:prog_name
+  if (parts_.size() != 2 && parts_.size() != 3 && parts_.size() != 4) {
     if (ap_->ignore_invalid)
       return SKIP;
 
-    return argument_count_error(1, 2);
+    return argument_count_error(1, 3);
+  }
+
+  if (parts_[1] == "bpf") {
+    ap_->target = parts_[1];
+    if (parts_.size() == 2) {
+      errs_ << "the 'bpf' variant of this probe requires a bpf program name "
+               "and optional bpf program id";
+      return INVALID;
+    } else if (parts_.size() == 3) {
+      ap_->func = parts_[2];
+    } else {
+      ap_->func = parts_[3];
+      if (parts_[2] != "*") {
+        auto uint_res = util::to_uint(parts_[2]);
+        if (!uint_res) {
+          errs_ << "bpf program id must be a number or '*'";
+          return INVALID;
+        }
+        ap_->bpf_prog_id = *uint_res;
+      }
+    }
+    return OK;
+  }
+
+  if (parts_.size() == 4) {
+    errs_ << "Only the 'bpf' variant of this probe supports 4 arguments";
+    return INVALID;
   }
 
   if (parts_.size() == 3) {
@@ -700,7 +713,7 @@ AttachPointParser::State AttachPointParser::fentry_parser()
     ap_->func = parts_[2];
   } else {
     ap_->func = parts_[1];
-    if (ap_->func.find('*') == std::string::npos) {
+    if (!util::has_wildcard(ap_->func)) {
       auto func_modules = bpftrace_.get_func_modules(ap_->func);
       if (func_modules.size() == 1)
         ap_->target = *func_modules.begin();
@@ -720,10 +733,6 @@ AttachPointParser::State AttachPointParser::fentry_parser()
       ap_->target = "*";
   }
 
-  if (ap_->func.find('*') != std::string::npos ||
-      ap_->target.find('*') != std::string::npos)
-    ap_->expansion = ExpansionType::FULL;
-
   return OK;
 }
 
@@ -738,19 +747,6 @@ AttachPointParser::State AttachPointParser::iter_parser()
     return INVALID;
   }
 
-  if (parts_[1].find('*') != std::string::npos) {
-    if (listing_) {
-      ap_->expansion = ExpansionType::FULL;
-    } else {
-      if (ap_->ignore_invalid)
-        return SKIP;
-
-      errs_ << ap_->provider << " probe type does not support wildcards"
-            << std::endl;
-      return INVALID;
-    }
-  }
-
   ap_->func = parts_[1];
 
   if (parts_.size() == 3)
@@ -760,19 +756,33 @@ AttachPointParser::State AttachPointParser::iter_parser()
 
 AttachPointParser::State AttachPointParser::raw_tracepoint_parser()
 {
-  if (parts_.size() != 2) {
+  if (parts_.size() != 2 && parts_.size() != 3) {
     if (ap_->ignore_invalid)
       return SKIP;
 
-    return argument_count_error(1);
+    return argument_count_error(2, 1);
   }
 
-  ap_->func = parts_[1];
-
-  if (has_wildcard(ap_->func))
-    ap_->expansion = ExpansionType::FULL;
+  if (parts_.size() == 3) {
+    ap_->target = parts_[1];
+    ap_->func = parts_[2];
+  } else {
+    // This is to maintain backwards compatibility with older scripts
+    // that couldn't include a target for a raw tracepoint.
+    ap_->target = "*";
+    ap_->func = parts_[1];
+  }
 
   return OK;
 }
 
+// Note: listing changes the parsing semantics for attach points
+Pass CreateParseAttachpointsPass(bool listing)
+{
+  return Pass::create("attachpoints", [listing](ASTContext &ast, BPFtrace &b) {
+    AttachPointParser ap_parser(ast, b, listing);
+    ap_parser.parse();
+  });
+}
+
 } // namespace bpftrace::ast
