@@ -1,22 +1,18 @@
 #pragma once
 
 #include <optional>
-#include <ostream>
 #include <sstream>
 #include <vector>
 
 #include "ast/ast.h"
+#include "ast/pass_manager.h"
 #include "bpftrace.h"
 
-namespace bpftrace {
-namespace ast {
+namespace bpftrace::ast {
 
 class AttachPointParser {
 public:
-  AttachPointParser(ASTContext &ctx,
-                    BPFtrace &bpftrace,
-                    std::ostream &sink,
-                    bool listing);
+  AttachPointParser(ASTContext &ctx, BPFtrace &bpftrace, bool listing);
   ~AttachPointParser() = default;
   int parse();
 
@@ -37,6 +33,7 @@ private:
   State lex_attachpoint(const AttachPoint &ap);
 
   State special_parser();
+  State benchmark_parser();
   State kprobe_parser(bool allow_offset = true);
   State kretprobe_parser();
   State uprobe_parser(bool allow_offset = true, bool allow_abs_addr = true);
@@ -52,6 +49,8 @@ private:
   State iter_parser();
   State raw_tracepoint_parser();
 
+  State frequency_parser();
+
   State argument_count_error(int expected,
                              std::optional<int> expected2 = std::nullopt);
   std::optional<uint64_t> stoull(const std::string &str);
@@ -59,7 +58,6 @@ private:
 
   ASTContext &ctx_;
   BPFtrace &bpftrace_;
-  std::ostream &sink_;
   AttachPoint *ap_{ nullptr }; // Non-owning pointer
   std::stringstream errs_;
   std::vector<std::string> parts_;
@@ -67,5 +65,7 @@ private:
   bool listing_;
 };
 
-} // namespace ast
-} // namespace bpftrace
+// The attachpoints are expanded in their own separate pass.
+Pass CreateParseAttachpointsPass(bool listing = false);
+
+} // namespace bpftrace::ast
