@@ -1,18 +1,16 @@
 #pragma once
 
-#include "functions.h"
-#include "types.h"
-
 #include <linux/bpf.h>
+#include <llvm/IR/DIBuilder.h>
+
+#include "ast/location.h"
+#include "types.h"
 
 namespace libbpf {
 #include "libbpf/bpf.h"
 } // namespace libbpf
 
-#include <llvm/IR/DIBuilder.h>
-
-namespace bpftrace {
-namespace ast {
+namespace bpftrace::ast {
 
 using namespace llvm;
 
@@ -20,11 +18,11 @@ class DIBuilderBPF : public DIBuilder {
 public:
   DIBuilderBPF(Module &module);
 
-  void createFunctionDebugInfo(llvm::Function &func,
-                               const SizedType &ret_type,
-                               const Struct &args,
-                               bool is_declaration = false);
-  void createProbeDebugInfo(llvm::Function &probe_func);
+  DILocalScope *createFunctionDebugInfo(llvm::Function &func,
+                                        const SizedType &ret_type,
+                                        const Struct &args,
+                                        bool is_declaration = false);
+  DILocalScope *createProbeDebugInfo(llvm::Function &probe_func);
 
   DIType *getInt8Ty();
   DIType *getInt16Ty();
@@ -39,6 +37,7 @@ public:
   DIType *GetType(const SizedType &stype, bool emit_codegen_types = true);
   DIType *CreateTupleType(const SizedType &stype);
   DIType *CreateMapStructType(const SizedType &stype);
+  DIType *CreateTSeriesStructType(const SizedType &stype);
   DIType *CreateByteArrayType(uint64_t num_bytes);
   DIType *createPointerMemberType(const std::string &name,
                                   uint64_t offset,
@@ -54,6 +53,9 @@ public:
                                              const SizedType &value_type);
   DIGlobalVariableExpression *createGlobalVariable(std::string_view name,
                                                    const SizedType &stype);
+  DILocation *createDebugLocation(llvm::LLVMContext &ctx,
+                                  DILocalScope *scope,
+                                  const ast::Location &loc);
 
   DIFile *file = nullptr;
 
@@ -71,5 +73,4 @@ private:
   std::unordered_map<std::string, DIType *> structs_;
 };
 
-} // namespace ast
-} // namespace bpftrace
+} // namespace bpftrace::ast
