@@ -1,31 +1,27 @@
 #include "codegen_resources.h"
 
-#include "ast/async_event_types.h"
-#include "struct.h"
 #include "types.h"
 
 namespace bpftrace::ast {
 
 CodegenResourceAnalyser::CodegenResourceAnalyser(
-    ASTContext &ctx,
     const ::bpftrace::Config &config)
-    : Visitor<CodegenResourceAnalyser>(ctx), config_(config)
+    : config_(config)
 {
 }
 
-CodegenResources CodegenResourceAnalyser::analyse()
+CodegenResources CodegenResourceAnalyser::analyse(Program &program)
 {
-  visit(ctx_.root);
+  visit(program);
   return std::move(resources_);
 }
 
 void CodegenResourceAnalyser::visit(Builtin &builtin)
 {
-  if (builtin.ident == "elapsed") {
+  if (builtin.ident == "__builtin_elapsed") {
     resources_.needs_elapsed_map = true;
   } else if (builtin.ident == "kstack" || builtin.ident == "ustack") {
-    resources_.stackid_maps.insert(
-        StackType{ .mode = config_.get(ConfigKeyStackMode::default_) });
+    resources_.stackid_maps.insert(StackType{ .mode = config_.stack_mode });
   }
 }
 
@@ -36,7 +32,7 @@ void CodegenResourceAnalyser::visit(Call &call)
   if (call.func == "join") {
     resources_.needs_join_map = true;
   } else if (call.func == "kstack" || call.func == "ustack") {
-    resources_.stackid_maps.insert(call.type.stack_type);
+    resources_.stackid_maps.insert(call.return_type.stack_type);
   }
 }
 
