@@ -1,15 +1,65 @@
-#include "field_analyser.h"
-
 #include <cassert>
-#include <iostream>
 
 #include "arch/arch.h"
+#include "ast/passes/field_analyser.h"
+#include "ast/passes/probe_expansion.h"
+#include "ast/visitor.h"
+#include "bpftrace.h"
 #include "dwarf_parser.h"
-#include "log.h"
 #include "probe_matcher.h"
+#include "util/strings.h"
+
+namespace libbpf {
+#include "libbpf/bpf.h"
+} // namespace libbpf
 
 namespace bpftrace::ast {
 
+namespace {
+
+class FieldAnalyser : public Visitor<FieldAnalyser> {
+public:
+  explicit FieldAnalyser(BPFtrace &bpftrace, ExpansionResult &expansions)
+      : bpftrace_(bpftrace), expansions_(expansions)
+  {
+  }
+
+  using Visitor<FieldAnalyser>::visit;
+  void visit(Identifier &identifier);
+  void visit(Builtin &builtin);
+  void visit(Map &map);
+  void visit(Variable &var);
+  void visit(FieldAccess &acc);
+  void visit(ArrayAccess &arr);
+  void visit(MapAccess &acc);
+  void visit(Cast &cast);
+  void visit(Sizeof &szof);
+  void visit(Offsetof &offof);
+  void visit(AssignMapStatement &assignment);
+  void visit(AssignVarStatement &assignment);
+  void visit(Unop &unop);
+  void visit(Probe &probe);
+  void visit(Subprog &subprog);
+
+private:
+  void resolve_args(Probe &probe);
+  void resolve_fields(SizedType &type);
+  void resolve_type(SizedType &type);
+
+  ProbeType probe_type_;
+  std::string attach_func_;
+  SizedType sized_type_;
+  BPFtrace &bpftrace_;
+  ExpansionResult &expansions_;
+  libbpf::bpf_prog_type prog_type_{ libbpf::BPF_PROG_TYPE_UNSPEC };
+  bool has_builtin_args_;
+  Probe *probe_ = nullptr;
+
+  std::map<std::string, SizedType> var_types_;
+};
+
+} // namespace
+
 void FieldAnalyser::visit(Identifier &identifier)
 {
   bpftrace_.btf_set_.insert(identifier.ident);
@@ -36,7 +86,7 @@ void FieldAnalyser::visit(Builtin &builtin)
     // make them resolved and available
     if (probe_type_ == ProbeType::iter)
       builtin_type = "struct bpf_iter__" + attach_func_;
-  } else if (builtin.ident == "curtask") {
+  } else if (builtin.ident == "__builtin_curtask") {
     builtin_type = "struct task_struct";
   } else if (builtin.ident == "args") {
     if (!probe_)
@@ -44,12 +94,12 @@ void FieldAnalyser::visit(Builtin &builtin)
     resolve_args(*probe_);
     has_builtin_args_ = true;
     return;
-  } else if (builtin.ident == "retval") {
+  } else if (builtin.ident == "__builtin_retval") {
     if (!probe_)
       return;
     resolve_args(*probe_);
 
-    auto arg = bpftrace_.structs.GetProbeArg(*probe_, RETVAL_FIELD_NAME);
+    const auto *arg = bpftrace_.structs.GetProbeArg(*probe_, RETVAL_FIELD_NAME);
     if (arg)
       sized_type_ = arg->type;
     return;
@@ -61,8 +111,6 @@ void FieldAnalyser::visit(Builtin &builtin)
 
 void FieldAnalyser::visit(Map &map)
 {
-  visit(map.key_expr);
-
   auto it = var_types_.find(map.ident);
   if (it != var_types_.end())
     sized_type_ = it->second;
@@ -82,7 +130,7 @@ void FieldAnalyser::visit(FieldAccess &acc)
   visit(acc.expr);
 
   if (has_builtin_args_) {
-    auto arg = bpftrace_.structs.GetProbeArg(*probe_, acc.field);
+    const auto *arg = bpftrace_.structs.GetProbeArg(*probe_, acc.field);
     if (arg)
       sized_type_ = arg->type;
 
@@ -118,28 +166,40 @@ void FieldAnalyser::visit(ArrayAccess &arr)
   }
 }
 
+void FieldAnalyser::visit(MapAccess &acc)
+{
+  visit(acc.key);
+  visit(acc.map); // Leaves sized_type_ as value type.
+}
+
 void FieldAnalyser::visit(Cast &cast)
 {
   visit(cast.expr);
-  resolve_type(cast.type);
+  resolve_type(cast.cast_type);
 }
 
 void FieldAnalyser::visit(Sizeof &szof)
 {
-  visit(szof.expr);
-  resolve_type(szof.argtype);
+  if (std::holds_alternative<SizedType>(szof.record)) {
+    resolve_type(std::get<SizedType>(szof.record));
+  } else {
+    visit(szof.record);
+  }
 }
 
 void FieldAnalyser::visit(Offsetof &offof)
 {
-  if (offof.expr)
-    visit(*offof.expr);
-  resolve_type(offof.record);
+  if (std::holds_alternative<SizedType>(offof.record)) {
+    resolve_type(std::get<SizedType>(offof.record));
+  } else {
+    visit(offof.record);
+  }
 }
 
 void FieldAnalyser::visit(AssignMapStatement &assignment)
 {
   visit(assignment.map);
+  visit(assignment.key);
   visit(assignment.expr);
   var_types_.emplace(assignment.map->ident, sized_type_);
 }
@@ -147,7 +207,7 @@ void FieldAnalyser::visit(AssignMapStatement &assignment)
 void FieldAnalyser::visit(AssignVarStatement &assignment)
 {
   visit(assignment.expr);
-  var_types_.emplace(assignment.var->ident, sized_type_);
+  var_types_.emplace(assignment.var()->ident, sized_type_);
 }
 
 void FieldAnalyser::visit(Unop &unop)
@@ -165,102 +225,104 @@ void FieldAnalyser::resolve_args(Probe &probe)
 {
   for (auto *ap : probe.attach_points) {
     // load probe arguments into a special record type "struct <probename>_args"
-    Struct probe_args;
+    std::shared_ptr<Struct> probe_args;
 
     auto probe_type = probetype(ap->provider);
     if (probe_type != ProbeType::fentry && probe_type != ProbeType::fexit &&
+        probe_type != ProbeType::rawtracepoint &&
         probe_type != ProbeType::uprobe)
       continue;
 
-    if (ap->expansion != ExpansionType::NONE) {
+    if (expansions_.get_expansion(*ap) != ExpansionType::NONE) {
       std::set<std::string> matches;
 
       // Find all the matches for the wildcard..
       try {
         matches = bpftrace_.probe_matcher_->get_matches_for_ap(*ap);
       } catch (const WildcardException &e) {
-        LOG(ERROR) << e.what();
+        probe.addError() << e.what();
         return;
       }
 
       // ... and check if they share same arguments.
 
-      Struct ap_args;
-      for (auto &match : matches) {
+      std::shared_ptr<Struct> ap_args;
+      for (const auto &match : matches) {
         // Both uprobes and fentry have a target (binary for uprobes, kernel
         // module for fentry).
         std::string func = match;
-        std::string target = erase_prefix(func);
+        std::string target = util::erase_prefix(func);
+        std::string err;
 
         // Trying to attach to multiple fentry. If some of them fails on
         // argument resolution, do not fail hard, just print a warning and
         // continue with other functions.
         if (probe_type == ProbeType::fentry || probe_type == ProbeType::fexit) {
-          std::string err;
-          auto maybe_ap_args = bpftrace_.btf_->resolve_args(
-              func, probe_type == ProbeType::fexit, err);
-          if (!maybe_ap_args.has_value()) {
-            LOG(WARNING) << "fentry:" << ap->func << ": " << err;
-            continue;
-          }
-          ap_args = std::move(*maybe_ap_args);
-        } else // uprobe
-        {
+          ap_args = bpftrace_.btf_->resolve_args(
+              func, probe_type == ProbeType::fexit, true, false, err);
+
+        } else if (probe_type == ProbeType::rawtracepoint) {
+          ap_args = bpftrace_.btf_->resolve_raw_tracepoint_args(func, err);
+        } else { // uprobe
           Dwarf *dwarf = bpftrace_.get_dwarf(target);
           if (dwarf)
             ap_args = dwarf->resolve_args(func);
           else
-            LOG(WARNING, ap->loc, err_) << "No debuginfo found for " << target;
+            ap->addWarning() << "No debuginfo found for " << target;
         }
 
-        if (probe_args.size == -1)
+        if (!ap_args) {
+          ap->addWarning() << probetypeName(probe_type) << ap->func << ": "
+                           << err;
+          continue;
+        }
+
+        if (!probe_args)
           probe_args = ap_args;
-        else if (ap_args != probe_args) {
-          LOG(ERROR, ap->loc, err_)
-              << "Probe has attach points with mixed arguments";
+        else if (*ap_args != *probe_args) {
+          ap->addError() << "Probe has attach points with mixed arguments";
           break;
         }
       }
     } else {
+      std::string err;
       // Resolving args for an explicit function failed, print an error and fail
       if (probe_type == ProbeType::fentry || probe_type == ProbeType::fexit) {
-        std::string err;
-        auto maybe_probe_args = bpftrace_.btf_->resolve_args(
-            ap->func, probe_type == ProbeType::fexit, err);
-        if (!maybe_probe_args.has_value()) {
-          LOG(ERROR, ap->loc, err_) << "fentry:" << ap->func << ": " << err;
-          return;
-        }
-        probe_args = std::move(*maybe_probe_args);
-      } else // uprobe
-      {
+        probe_args = bpftrace_.btf_->resolve_args(
+            ap->func, probe_type == ProbeType::fexit, true, false, err);
+
+      } else if (probe_type == ProbeType::rawtracepoint) {
+        probe_args = bpftrace_.btf_->resolve_raw_tracepoint_args(ap->func, err);
+      } else { // uprobe
         Dwarf *dwarf = bpftrace_.get_dwarf(ap->target);
-        if (dwarf)
+        if (dwarf) {
           probe_args = dwarf->resolve_args(ap->func);
-        else {
-          LOG(WARNING, ap->loc, err_)
-              << "No debuginfo found for " << ap->target;
+        } else {
+          ap->addWarning() << "No debuginfo found for " << ap->target;
         }
-        if (static_cast<int>(probe_args.fields.size()) >
-            (arch::max_arg() + 1)) {
-          LOG(ERROR, ap->loc, err_) << "\'args\' builtin is not supported for "
-                                       "probes with stack-passed arguments.";
+        if (probe_args &&
+            probe_args->fields.size() >= arch::Host::arguments().size()) {
+          ap->addError() << "\'args\' builtin is not supported for "
+                         << "probes with stack-passed arguments.";
         }
       }
+
+      if (!probe_args) {
+        ap->addError() << probetypeName(probe_type) << ap->func << ": " << err;
+        return;
+      }
     }
 
     // check if we already stored arguments for this probe
     auto args = bpftrace_.structs.Lookup(probe.args_typename()).lock();
-    if (args && *args != probe_args) {
+    if (args && *args != *probe_args) {
       // we did, and it's different...trigger the error
-      LOG(ERROR, ap->loc, err_)
-          << "Probe has attach points with mixed arguments";
+      ap->addError() << "Probe has attach points with mixed arguments";
     } else {
       // store/save args for each ap for later processing
       bpftrace_.structs.Add(probe.args_typename(), std::move(probe_args));
     }
   }
-  return;
 }
 
 void FieldAnalyser::resolve_fields(SizedType &type)
@@ -287,7 +349,7 @@ void FieldAnalyser::resolve_type(SizedType &type)
     inner_type = inner_type->GetPointeeTy();
   if (!inner_type->IsRecordTy())
     return;
-  auto name = inner_type->GetName();
+  const auto &name = inner_type->GetName();
 
   if (probe_) {
     for (auto &ap : probe_->attach_points)
@@ -324,17 +386,14 @@ void FieldAnalyser::visit(Subprog &subprog)
   visit(subprog.stmts);
 }
 
-int FieldAnalyser::analyse()
+Pass CreateFieldAnalyserPass()
 {
-  visit(ctx_.root);
+  auto fn = [](ASTContext &ast, BPFtrace &b, ExpansionResult &expansions) {
+    FieldAnalyser analyser(b, expansions);
+    analyser.visit(ast.root);
+  };
 
-  std::string errors = err_.str();
-  if (!errors.empty()) {
-    out_ << errors;
-    return 1;
-  }
-
-  return 0;
-}
+  return Pass::create("FieldAnalyser", fn);
+};
 
 } // namespace bpftrace::ast
