@@ -2,7 +2,6 @@
 
 #include <cctype>
 #include <iomanip>
-#include <regex>
 #include <sstream>
 
 #include "ast/ast.h"
@@ -10,19 +9,12 @@
 
 namespace bpftrace::ast {
 
-void Printer::print()
-{
-  ++depth_;
-  visit(ctx_.root);
-  --depth_;
-}
-
 std::string Printer::type(const SizedType &ty)
 {
   if (ty.IsNoneTy())
     return "";
   std::stringstream buf;
-  buf << " :: [" << ty;
+  buf << " :: [" << typestr(ty, true);
   if (ty.IsCtxAccess())
     buf << ", ctx: 1";
   if (ty.GetAS() != AddrSpace::none)
@@ -34,23 +26,32 @@ std::string Printer::type(const SizedType &ty)
 void Printer::visit(Integer &integer)
 {
   std::string indent(depth_, ' ');
-  out_ << indent << "int: " << integer.n << type(integer.type) << std::endl;
+  out_ << indent << "int: " << integer.value << type(integer.integer_type)
+       << std::endl;
+}
+
+void Printer::visit(NegativeInteger &integer)
+{
+  std::string indent(depth_, ' ');
+  out_ << indent << "negative int: " << integer.value << std::endl;
+}
+
+void Printer::visit(Boolean &boolean)
+{
+  std::string indent(depth_, ' ');
+  out_ << indent << "bool: " << (boolean.value ? "true" : "false") << std::endl;
 }
 
 void Printer::visit(PositionalParameter &param)
 {
   std::string indent(depth_, ' ');
+  out_ << indent << "param: $" << param.n << std::endl;
+}
 
-  switch (param.ptype) {
-    case PositionalParameterType::positional:
-      out_ << indent << "param: $" << param.n << type(param.type) << std::endl;
-      break;
-    case PositionalParameterType::count:
-      out_ << indent << "param: $#" << type(param.type) << std::endl;
-      break;
-    default:
-      break;
-  }
+void Printer::visit([[maybe_unused]] PositionalParameterCount &param)
+{
+  std::string indent(depth_, ' ');
+  out_ << indent << "param: $#" << std::endl;
 }
 
 void Printer::visit(String &string)
@@ -58,7 +59,7 @@ void Printer::visit(String &string)
   std::string indent(depth_, ' ');
   std::stringstream ss;
 
-  for (char c : string.str) {
+  for (char c : string.value) {
     // the argument of isprint() must be an unsigned char or EOF
     int code = static_cast<unsigned char>(c);
     if (std::isprint(code)) {
@@ -80,33 +81,28 @@ void Printer::visit(String &string)
     }
   }
 
-  out_ << indent << "string: " << ss.str() << type(string.type) << std::endl;
-}
-
-void Printer::visit(StackMode &mode)
-{
-  std::string indent(depth_, ' ');
-  out_ << indent << "stack_mode: " << mode.mode << type(mode.type) << std::endl;
+  out_ << indent << "string: " << ss.str() << std::endl;
 }
 
 void Printer::visit(Builtin &builtin)
 {
   std::string indent(depth_, ' ');
-  out_ << indent << "builtin: " << builtin.ident << type(builtin.type)
+  out_ << indent << "builtin: " << builtin.ident << type(builtin.builtin_type)
        << std::endl;
 }
 
 void Printer::visit(Identifier &identifier)
 {
   std::string indent(depth_, ' ');
-  out_ << indent << "identifier: " << identifier.ident << type(identifier.type)
-       << std::endl;
+  out_ << indent << "identifier: " << identifier.ident
+       << type(identifier.ident_type) << std::endl;
 }
 
 void Printer::visit(Call &call)
 {
   std::string indent(depth_, ' ');
-  out_ << indent << "call: " << call.func << type(call.type) << std::endl;
+  out_ << indent << "call: " << call.func << type(call.return_type)
+       << std::endl;
 
   ++depth_;
   visit(call.vargs);
@@ -116,26 +112,26 @@ void Printer::visit(Call &call)
 void Printer::visit(Sizeof &szof)
 {
   std::string indent(depth_, ' ');
-  out_ << indent << "sizeof: " << type(szof.type) << std::endl;
+  out_ << indent << "sizeof: " << std::endl;
 
   ++depth_;
-  visit(szof.expr);
+  visit(szof.record);
   --depth_;
 }
 
 void Printer::visit(Offsetof &offof)
 {
   std::string indent(depth_, ' ');
-  out_ << indent << "offsetof: " << type(offof.type) << std::endl;
+  out_ << indent << "offsetof: " << std::endl;
 
   ++depth_;
   std::string indentParam(depth_, ' ');
 
   // Print the args
-  if (offof.expr) {
-    visit(*offof.expr);
+  if (std::holds_alternative<Expression>(offof.record)) {
+    visit(std::get<Expression>(offof.record));
   } else {
-    out_ << indentParam << offof.record << std::endl;
+    out_ << indentParam << std::get<SizedType>(offof.record) << std::endl;
   }
 
   for (const auto &field : offof.field) {
@@ -144,26 +140,69 @@ void Printer::visit(Offsetof &offof)
   --depth_;
 }
 
-void Printer::visit(Map &map)
+void Printer::visit(MapDeclStatement &decl)
 {
   std::string indent(depth_, ' ');
-  out_ << indent << "map: " << map.ident << type(map.type) << std::endl;
+  out_ << indent << "map decl: " << decl.ident << std::endl;
 
   ++depth_;
-  visit(map.key_expr);
+  std::string indentType(depth_, ' ');
+  out_ << indentType << "bpf type: " << decl.bpf_type << std::endl;
+  out_ << indentType << "max entries: " << decl.max_entries << std::endl;
+  --depth_;
+}
+
+void Printer::visit(Map &map)
+{
+  // Use a slightly customized format for the map type here, since it is never
+  // going to be marked as `is_ctx` or have an associated address space.
+  std::string indent(depth_, ' ');
+  out_ << indent << "map: " << map.ident;
+  if (!map.key_type.IsNoneTy() || !map.value_type.IsNoneTy()) {
+    out_ << " :: ";
+  }
+  if (!map.key_type.IsNoneTy()) {
+    out_ << "[" << typestr(map.key_type, true) << "]";
+  }
+  if (!map.value_type.IsNoneTy()) {
+    out_ << typestr(map.value_type, true);
+  }
+  out_ << std::endl;
+}
+
+void Printer::visit(MapAddr &map_addr)
+{
+  std::string indent(depth_, ' ');
+
+  out_ << indent << "&" << std::endl;
+
+  ++depth_;
+  visit(map_addr.map);
   --depth_;
 }
 
 void Printer::visit(Variable &var)
 {
   std::string indent(depth_, ' ');
-  out_ << indent << "variable: " << var.ident << type(var.type) << std::endl;
+  out_ << indent << "variable: " << var.ident << type(var.var_type)
+       << std::endl;
+}
+
+void Printer::visit(VariableAddr &var_addr)
+{
+  std::string indent(depth_, ' ');
+
+  out_ << indent << "&" << std::endl;
+
+  ++depth_;
+  visit(var_addr.var);
+  --depth_;
 }
 
 void Printer::visit(Binop &binop)
 {
   std::string indent(depth_, ' ');
-  out_ << indent << opstr(binop) << type(binop.type) << std::endl;
+  out_ << indent << opstr(binop) << type(binop.result_type) << std::endl;
 
   ++depth_;
   visit(binop.left);
@@ -174,7 +213,7 @@ void Printer::visit(Binop &binop)
 void Printer::visit(Unop &unop)
 {
   std::string indent(depth_, ' ');
-  out_ << indent << opstr(unop) << type(unop.type) << std::endl;
+  out_ << indent << opstr(unop) << type(unop.result_type) << std::endl;
 
   ++depth_;
   visit(unop.expr);
@@ -184,7 +223,7 @@ void Printer::visit(Unop &unop)
 void Printer::visit(Ternary &ternary)
 {
   std::string indent(depth_, ' ');
-  out_ << indent << "?:" << type(ternary.type) << std::endl;
+  out_ << indent << "?:" << type(ternary.result_type) << std::endl;
 
   ++depth_;
   visit(ternary.cond);
@@ -196,22 +235,19 @@ void Printer::visit(Ternary &ternary)
 void Printer::visit(FieldAccess &acc)
 {
   std::string indent(depth_, ' ');
-  out_ << indent << "." << type(acc.type) << std::endl;
+  out_ << indent << "." << type(acc.field_type) << std::endl;
 
   ++depth_;
   visit(acc.expr);
   --depth_;
 
-  if (acc.field.size())
-    out_ << indent << " " << acc.field << std::endl;
-  else
-    out_ << indent << " " << acc.index << std::endl;
+  out_ << indent << " " << acc.field << std::endl;
 }
 
 void Printer::visit(ArrayAccess &arr)
 {
   std::string indent(depth_, ' ');
-  out_ << indent << "[]" << type(arr.type) << std::endl;
+  out_ << indent << "[]" << type(arr.element_type) << std::endl;
 
   ++depth_;
   visit(arr.expr);
@@ -219,10 +255,33 @@ void Printer::visit(ArrayAccess &arr)
   --depth_;
 }
 
+void Printer::visit(TupleAccess &acc)
+{
+  std::string indent(depth_, ' ');
+  out_ << indent << "." << type(acc.element_type) << std::endl;
+
+  ++depth_;
+  visit(acc.expr);
+  --depth_;
+
+  out_ << indent << " " << acc.index << std::endl;
+}
+
+void Printer::visit(MapAccess &acc)
+{
+  std::string indent(depth_, ' ');
+  out_ << indent << "[]" << type(acc.type()) << std::endl;
+
+  ++depth_;
+  visit(acc.map);
+  visit(acc.key);
+  --depth_;
+}
+
 void Printer::visit(Cast &cast)
 {
   std::string indent(depth_, ' ');
-  out_ << indent << "(" << cast.type << ")" << std::endl;
+  out_ << indent << "(" << cast.type() << ")" << std::endl;
 
   ++depth_;
   visit(cast.expr);
@@ -232,7 +291,7 @@ void Printer::visit(Cast &cast)
 void Printer::visit(Tuple &tuple)
 {
   std::string indent(depth_, ' ');
-  out_ << indent << "tuple:" << type(tuple.type) << std::endl;
+  out_ << indent << "tuple:" << type(tuple.type()) << std::endl;
 
   ++depth_;
   visit(tuple.elems);
@@ -244,7 +303,7 @@ void Printer::visit(ExprStatement &expr)
   visit(expr.expr);
 }
 
-void Printer::visit(AssignMapStatement &assignment)
+void Printer::visit(AssignScalarMapStatement &assignment)
 {
   std::string indent(depth_, ' ');
   out_ << indent << "=" << std::endl;
@@ -255,12 +314,26 @@ void Printer::visit(AssignMapStatement &assignment)
   --depth_;
 }
 
+void Printer::visit(AssignMapStatement &assignment)
+{
+  std::string indent(depth_, ' ');
+  out_ << indent << "=" << std::endl;
+
+  ++depth_;
+  visit(assignment.map);
+  ++depth_;
+  visit(assignment.key);
+  --depth_;
+  visit(assignment.expr);
+  --depth_;
+}
+
 void Printer::visit(AssignVarStatement &assignment)
 {
   std::string indent(depth_, ' ');
 
-  if (assignment.var_decl_stmt) {
-    visit(assignment.var_decl_stmt);
+  if (std::holds_alternative<VarDeclStatement *>(assignment.var_decl)) {
+    visit(std::get<VarDeclStatement *>(assignment.var_decl));
     ++depth_;
     visit(assignment.expr);
     --depth_;
@@ -268,7 +341,7 @@ void Printer::visit(AssignVarStatement &assignment)
     out_ << indent << "=" << std::endl;
 
     ++depth_;
-    visit(assignment.var);
+    visit(std::get<Variable *>(assignment.var_decl));
     visit(assignment.expr);
     --depth_;
   }
@@ -281,15 +354,31 @@ void Printer::visit(AssignConfigVarStatement &assignment)
 
   ++depth_;
   std::string indentVar(depth_, ' ');
-  out_ << indentVar << "config var: " << assignment.config_var << std::endl;
-  visit(assignment.expr);
+  out_ << indentVar << "var: " << assignment.var << std::endl;
+  std::visit(
+      [&](auto &v) {
+        if constexpr (std::is_same_v<std::decay_t<decltype(v)>, std::string>) {
+          out_ << indentVar << "string: " << v << std::endl;
+        } else if constexpr (std::is_same_v<std::decay_t<decltype(v)>, bool>) {
+          out_ << indentVar << "bool: " << (v ? "true" : "false") << std::endl;
+        } else {
+          out_ << indentVar << "int: " << v << std::endl;
+        }
+      },
+      assignment.value);
   --depth_;
 }
 
 void Printer::visit(VarDeclStatement &decl)
 {
   std::string indent(depth_, ' ');
-  out_ << indent << "decl" << std::endl;
+
+  if (decl.type) {
+    out_ << indent << "decl" << type(*decl.type) << std::endl;
+  } else {
+    out_ << indent << "decl" << std::endl;
+  }
+
   ++depth_;
   visit(decl.var);
   --depth_;
@@ -345,6 +434,21 @@ void Printer::visit(While &while_block)
   visit(while_block.block);
 }
 
+void Printer::visit(Range &range)
+{
+  std::string indent(depth_, ' ');
+
+  out_ << indent << "start\n";
+  ++depth_;
+  visit(range.start);
+  --depth_;
+
+  out_ << indent << "end\n";
+  ++depth_;
+  visit(range.end);
+  --depth_;
+}
+
 void Printer::visit(For &for_loop)
 {
   std::string indent(depth_, ' ');
@@ -362,11 +466,7 @@ void Printer::visit(For &for_loop)
   out_ << indent << " decl\n";
   ++depth_;
   visit(for_loop.decl);
-  --depth_;
-
-  out_ << indent << " expr\n";
-  ++depth_;
-  visit(for_loop.expr);
+  visit(for_loop.iterable);
   --depth_;
 
   out_ << indent << " stmts\n";
@@ -423,28 +523,44 @@ void Printer::visit(Probe &probe)
   --depth_;
 }
 
+void Printer::visit(SubprogArg &arg)
+{
+  std::string indent(depth_, ' ');
+
+  ++depth_;
+  out_ << indent << arg.name << type(arg.type) << std::endl;
+  --depth_;
+}
+
 void Printer::visit(Subprog &subprog)
 {
   std::string indent(depth_, ' ');
-  out_ << indent << subprog.name() << ": " << subprog.return_type;
-
-  out_ << "(";
-  for (size_t i = 0; i < subprog.args.size(); i++) {
-    auto &arg = subprog.args.at(i);
-    out_ << arg->name() << " : " << arg->type;
-    if (i < subprog.args.size() - 1)
-      out_ << ", ";
-  }
-  out_ << ")" << std::endl;
+  out_ << indent << "subprog: " << subprog.name << type(subprog.return_type)
+       << std::endl;
 
   ++depth_;
+
+  if (!subprog.args.empty()) {
+    ++depth_;
+    out_ << indent << " args" << std::endl;
+    visit(subprog.args);
+    --depth_;
+  }
+
   visit(subprog.stmts);
+
   --depth_;
 }
 
+void Printer::visit(Import &imp)
+{
+  std::string indent(depth_, ' ');
+  out_ << indent << "import " << imp.name << std::endl;
+}
+
 void Printer::visit(Program &program)
 {
-  if (program.c_definitions.size() > 0)
+  if (!program.c_definitions.empty())
     out_ << program.c_definitions << std::endl;
 
   std::string indent(depth_, ' ');
@@ -454,6 +570,14 @@ void Printer::visit(Program &program)
   visit(program.config);
   --depth_;
 
+  ++depth_;
+  visit(program.imports);
+  --depth_;
+
+  ++depth_;
+  visit(program.map_decls);
+  --depth_;
+
   ++depth_;
   visit(program.functions);
   visit(program.probes);
