@@ -7,25 +7,32 @@
 #include <cstdint>
 #include <cstdio>
 #include <fcntl.h>
+#include <filesystem>
 #include <fstream>
 #include <sys/stat.h>
 #include <unistd.h>
 
 #include "bpf_assembler.h"
 #include "btf.h"
-#include "debugfs.h"
 #include "dwarf_parser.h"
-#include "probe_matcher.h"
-#include "tracefs.h"
-#include "utils.h"
+#include "tracefs/tracefs.h"
+#include "util/kernel.h"
+#include "util/strings.h"
+
+#define ARRAY_SIZE(x) (sizeof(x) / sizeof((x)[0]))
 
 namespace bpftrace {
 
+using util::KernelVersionMethod;
+
 int BPFnofeature::parse(const char* str)
 {
-  for (auto feat : split_string(str, ',')) {
+  for (auto feat : util::split_string(str, ',')) {
+    // Remember to update bpftrace.adoc!
     if (feat == "kprobe_multi") {
       kprobe_multi_ = true;
+    } else if (feat == "kprobe_session") {
+      kprobe_session_ = true;
     } else if (feat == "uprobe_multi") {
       uprobe_multi_ = true;
     } else {
@@ -46,12 +53,14 @@ static bool try_load_(const char* name,
                       size_t logbuf_size,
                       int* outfd = nullptr)
 {
-  const KernelVersionMethod methods[] = { vDSO, UTS, File };
+  const KernelVersionMethod methods[] = { KernelVersionMethod::vDSO,
+                                          KernelVersionMethod::UTS,
+                                          KernelVersionMethod::File };
 
   for (KernelVersionMethod method : methods) {
     auto version = kernel_version(method);
 
-    if (method != vDSO && !version) {
+    if (method != KernelVersionMethod::vDSO && !version) {
       // Recent kernels don't check the version so we should try to call
       // bpf_prog_load during first iteration even if we failed to determine
       // the version. We should not do that in subsequent iterations to avoid
@@ -367,18 +376,50 @@ bool BPFfeature::has_d_path()
   return *has_d_path_;
 }
 
-bool BPFfeature::has_uprobe_refcnt()
+bool try_create_link(libbpf::bpf_prog_type prog_type,
+                     const std::string_view prog_name,
+                     libbpf::bpf_attach_type expected_attach_type,
+                     const bpf_link_create_opts& link_opts,
+                     std::optional<int> expected_err)
 {
-  if (has_uprobe_refcnt_.has_value())
-    return *has_uprobe_refcnt_;
+  bool result = false;
 
-  std::error_code ec;
-  std::filesystem::path path{
-    "/sys/bus/event_source/devices/uprobe/format/ref_ctr_offset"
+  BPFTRACE_LIBBPF_OPTS(
+      bpf_prog_load_opts,
+      load_opts,
+      .expected_attach_type = static_cast<enum ::bpf_attach_type>(
+          expected_attach_type));
+
+  struct bpf_insn insns[] = {
+    BPF_MOV64_IMM(BPF_REG_0, 0),
+    BPF_EXIT_INSN(),
   };
-  has_uprobe_refcnt_ = std::filesystem::exists(path, ec);
 
-  return *has_uprobe_refcnt_;
+  int progfd = bpf_prog_load(static_cast<::bpf_prog_type>(prog_type),
+                             prog_name.data(),
+                             "GPL",
+                             reinterpret_cast<struct bpf_insn*>(insns),
+                             ARRAY_SIZE(insns),
+                             &load_opts);
+
+  if (progfd < 0)
+    return false;
+
+  int linkfd = bpf_link_create(progfd,
+                               0,
+                               static_cast<enum ::bpf_attach_type>(
+                                   expected_attach_type),
+                               &link_opts);
+
+  result = expected_err.has_value() ? linkfd < 0 && -errno == *expected_err
+                                    : linkfd >= 0;
+
+  if (linkfd >= 0) {
+    close(linkfd);
+  }
+  close(progfd);
+
+  return result;
 }
 
 bool BPFfeature::has_kprobe_multi()
@@ -392,104 +433,65 @@ bool BPFfeature::has_kprobe_multi()
   }
 
   const char* sym = "ksys_read";
+
   BPFTRACE_LIBBPF_OPTS(bpf_link_create_opts, link_opts);
-  int progfd, linkfd = -1;
-
-  struct bpf_insn insns[] = {
-    BPF_MOV64_IMM(BPF_REG_0, 0),
-    BPF_EXIT_INSN(),
-  };
-
   link_opts.kprobe_multi.syms = &sym;
   link_opts.kprobe_multi.cnt = 1;
 
-  BPFTRACE_LIBBPF_OPTS(bpf_prog_load_opts, load_opts);
-  load_opts.expected_attach_type = static_cast<enum ::bpf_attach_type>(
-      libbpf::BPF_TRACE_KPROBE_MULTI);
-
-  progfd = bpf_prog_load(static_cast<::bpf_prog_type>(
-                             libbpf::BPF_PROG_TYPE_KPROBE),
-                         sym,
-                         "GPL",
-                         reinterpret_cast<struct bpf_insn*>(insns),
-                         ARRAY_SIZE(insns),
-                         &load_opts);
-
-  if (progfd >= 0) {
-    linkfd = bpf_link_create(progfd,
-                             0,
-                             static_cast<enum ::bpf_attach_type>(
-                                 libbpf::BPF_TRACE_KPROBE_MULTI),
-                             &link_opts);
-  }
-
-  has_kprobe_multi_ = linkfd >= 0;
-
-  if (linkfd >= 0) {
-    close(linkfd);
-  }
-  if (progfd >= 0) {
-    close(progfd);
-  }
+  has_kprobe_multi_ = try_create_link(libbpf::BPF_PROG_TYPE_KPROBE,
+                                      sym,
+                                      libbpf::BPF_TRACE_KPROBE_MULTI,
+                                      link_opts,
+                                      std::nullopt);
   return *has_kprobe_multi_;
 }
 
+bool BPFfeature::has_kprobe_session()
+{
+  if (has_kprobe_session_.has_value())
+    return *has_kprobe_session_;
+
+  if (no_feature_.kprobe_session_) {
+    has_kprobe_session_ = false;
+    return *has_kprobe_session_;
+  }
+
+  const char* sym = "ksys_read";
+
+  BPFTRACE_LIBBPF_OPTS(bpf_link_create_opts, link_opts);
+  link_opts.kprobe_multi.syms = &sym;
+  link_opts.kprobe_multi.cnt = 1;
+
+  has_kprobe_session_ = try_create_link(libbpf::BPF_PROG_TYPE_KPROBE,
+                                        sym,
+                                        libbpf::BPF_TRACE_KPROBE_SESSION,
+                                        link_opts,
+                                        std::nullopt);
+  return *has_kprobe_session_;
+}
+
 bool BPFfeature::has_uprobe_multi()
 {
   if (has_uprobe_multi_.has_value())
     return *has_uprobe_multi_;
 
-#if defined(HAVE_LIBBPF_UPROBE_MULTI)
+#ifdef HAVE_LIBBPF_UPROBE_MULTI
   if (no_feature_.uprobe_multi_) {
     has_uprobe_multi_ = false;
     return *has_uprobe_multi_;
   }
 
-  BPFTRACE_LIBBPF_OPTS(
-      bpf_prog_load_opts,
-      load_opts,
-      .expected_attach_type = static_cast<enum ::bpf_attach_type>(
-          libbpf::BPF_TRACE_UPROBE_MULTI), );
+  BPFTRACE_LIBBPF_OPTS(bpf_link_create_opts, link_opts);
+  const unsigned long offset = 0;
+  link_opts.uprobe_multi.path = "/";
+  link_opts.uprobe_multi.offsets = &offset;
+  link_opts.uprobe_multi.cnt = 1;
 
-  int err = 0, progfd, linkfd = -1;
-
-  struct bpf_insn insns[] = {
-    BPF_MOV64_IMM(BPF_REG_0, 0),
-    BPF_EXIT_INSN(),
-  };
-
-  progfd = bpf_prog_load(static_cast<::bpf_prog_type>(
-                             libbpf::BPF_PROG_TYPE_KPROBE),
-                         "uprobe_multi",
-                         "GPL",
-                         reinterpret_cast<struct bpf_insn*>(insns),
-                         ARRAY_SIZE(insns),
-                         &load_opts);
-
-  if (progfd >= 0) {
-    BPFTRACE_LIBBPF_OPTS(bpf_link_create_opts, link_opts);
-    const unsigned long offset = 0;
-
-    link_opts.uprobe_multi.path = "/";
-    link_opts.uprobe_multi.offsets = &offset;
-    link_opts.uprobe_multi.cnt = 1;
-
-    linkfd = bpf_link_create(progfd,
-                             0,
-                             static_cast<enum ::bpf_attach_type>(
-                                 libbpf::BPF_TRACE_UPROBE_MULTI),
-                             &link_opts);
-    err = -errno;
-  }
-
-  has_uprobe_multi_ = linkfd < 0 && err == -EBADF;
-
-  if (linkfd >= 0) {
-    close(linkfd);
-  }
-  if (progfd >= 0) {
-    close(progfd);
-  }
+  has_uprobe_multi_ = try_create_link(libbpf::BPF_PROG_TYPE_KPROBE,
+                                      "uprobe_multi",
+                                      libbpf::BPF_TRACE_UPROBE_MULTI,
+                                      link_opts,
+                                      -EBADF);
 #else
   has_uprobe_multi_ = false;
 #endif                       // HAVE_LIBBPF_UPROBE_MULTI
@@ -567,15 +569,11 @@ std::string BPFfeature::report()
   auto to_str = [](bool f) -> std::string { return f ? "yes" : "no"; };
 
   std::vector<std::pair<std::string, std::string>> helpers = {
-    { "probe_read", to_str(has_helper_probe_read()) },
-    { "probe_read_str", to_str(has_helper_probe_read_str()) },
     { "probe_read_user", to_str(has_helper_probe_read_user()) },
     { "probe_read_user_str", to_str(has_helper_probe_read_user_str()) },
     { "probe_read_kernel", to_str(has_helper_probe_read_kernel()) },
     { "probe_read_kernel_str", to_str(has_helper_probe_read_kernel_str()) },
-    { "get_current_cgroup_id", to_str(has_helper_get_current_cgroup_id()) },
     { "send_signal", to_str(has_helper_send_signal()) },
-    { "override_return", to_str(has_helper_override_return()) },
     { "get_boot_ns", to_str(has_helper_ktime_get_boot_ns()) },
     { "dpath", to_str(has_d_path()) },
     { "skboutput", to_str(has_skb_output()) },
@@ -590,11 +588,8 @@ std::string BPFfeature::report()
   std::vector<std::pair<std::string, std::string>> features = {
     { "Instruction limit", std::to_string(instruction_limit()) },
     { "btf", to_str(has_btf()) },
-    { "module btf", to_str(has_module_btf()) },
-    { "Kernel DWARF", to_str(has_kernel_dwarf()) },
+    { "module btf", to_str(btf_.has_module_btf()) },
     { "map batch", to_str(has_map_batch()) },
-    // Depends on BCC's bpf_attach_uprobe refcount feature
-    { "uprobe refcount", to_str(has_uprobe_refcnt()) }
   };
 
   std::vector<std::pair<std::string, std::string>> map_types = {
@@ -602,7 +597,6 @@ std::string BPFfeature::report()
     { "array", to_str(has_map_array()) },
     { "percpu array", to_str(has_map_percpu_array()) },
     { "stack_trace", to_str(has_map_stack_trace()) },
-    { "perf_event_array", to_str(has_map_perf_event_array()) },
     { "ringbuf", to_str(has_map_ringbuf()) }
   };
 
@@ -613,6 +607,7 @@ std::string BPFfeature::report()
     { "fentry", to_str(has_fentry()) },
     { "kprobe_multi", to_str(has_kprobe_multi()) },
     { "uprobe_multi", to_str(has_uprobe_multi()) },
+    { "kprobe_session", to_str(has_kprobe_session()) },
     { "iter", to_str(has_iter("task")) }
   };
 
@@ -662,39 +657,6 @@ bool BPFfeature::has_fentry()
   return has_prog_fentry() && btf_.has_data();
 }
 
-bool BPFfeature::has_module_btf()
-{
-  if (has_module_btf_.has_value())
-    return *has_module_btf_;
-
-  char name[64];
-  struct bpf_btf_info info = {};
-  info.name = reinterpret_cast<uintptr_t>(name);
-  info.name_len = sizeof(name);
-  __u32 id = 0, info_len = sizeof(info);
-  int err = 0, fd = -1;
-
-  err = bpf_btf_get_next_id(id, &id);
-  if (err)
-    goto not_support;
-
-  fd = bpf_btf_get_fd_by_id(id);
-  if (fd < 0)
-    goto not_support;
-
-  err = bpf_obj_get_info_by_fd(fd, &info, &info_len);
-  close(fd);
-  if (err)
-    goto not_support;
-
-  has_module_btf_ = true;
-  return *has_module_btf_;
-
-not_support:
-  has_module_btf_ = false;
-  return *has_module_btf_;
-}
-
 bool BPFfeature::has_iter(std::string name)
 {
   auto tracing_name = "bpf_iter_" + name;
@@ -703,28 +665,6 @@ bool BPFfeature::has_iter(std::string name)
                           libbpf::BPF_TRACE_ITER);
 }
 
-bool BPFfeature::has_kernel_dwarf()
-{
-#ifndef HAVE_LIBLLDB
-  return false;
-#endif
-
-  auto vmlinux = find_vmlinux();
-  if (!vmlinux.has_value())
-    return false;
-
-  // WARNING: we are not passing a pointer to BPFtrace, so we can only use:
-  // * Dwarf::has_debug_info
-  // * Dwarf::get_function_locations
-  // * Dwarf::get_function_params
-  // Otherwise, Dwarf will try to use the BPFtrace pointer and will segfault.
-  auto dwarf = Dwarf::GetFromBinary(nullptr, vmlinux.value());
-  if (!dwarf)
-    return false;
-
-  return dwarf->has_debug_info();
-}
-
 bool BPFfeature::has_kernel_func(Kfunc kfunc)
 {
   if (!has_btf())
