@@ -21,8 +21,8 @@ std::string BuildInfo::report()
 #else
       << "no" << std::endl;
 #endif
-  buf << "  liblldb (DWARF support): "
-#ifdef HAVE_LIBLLDB
+  buf << "  libdw (DWARF support): "
+#ifdef HAVE_LIBDW
       << "yes" << std::endl;
 #else
       << "no" << std::endl;
@@ -33,6 +33,12 @@ std::string BuildInfo::report()
 #else
       << "no" << std::endl;
 #endif
+  buf << "  blazesym (advanced symbolization): "
+#ifdef HAVE_BLAZESYM
+      << "yes" << std::endl;
+#else
+      << "no" << std::endl;
+#endif
 
   return buf.str();
 }
