@@ -4,9 +4,9 @@
 #include <iostream>
 #include <optional>
 #include <sstream>
+#include <string>
 #include <unordered_map>
-
-#include "location.hh"
+#include <vector>
 
 namespace bpftrace {
 
@@ -31,41 +31,33 @@ enum class LogType
 
 class Log {
 public:
-  static Log& get();
-  void take_input(LogType type,
-                  const std::optional<location>& loc,
-                  std::ostream& out,
-                  std::string&& input);
-  inline void set_source(std::string_view filename, std::string_view source)
-  {
-    src_ = source;
-    filename_ = filename;
-  }
-  inline const std::string& get_source()
-  {
-    return src_;
-  }
-  const std::string get_source_line(unsigned int n);
-
-  // Can only construct with get()
   Log(const Log& other) = delete;
   Log& operator=(const Log& other) = delete;
+  Log(Log&& other) = delete;
+  Log& operator=(Log&& other) = delete;
 
-  inline void enable(LogType type)
+  static Log& get();
+
+  void take_input(LogType type,
+                  std::optional<std::string>&& source_location,
+                  std::optional<std::vector<std::string>>&& source_context,
+                  std::ostream& out,
+                  std::string&& msg);
+
+  void enable(LogType type)
   {
     enabled_map_[type] = true;
   }
-  inline void disable(LogType type)
+  void disable(LogType type)
   {
     assert(type != LogType::BUG && type != LogType::ERROR);
     enabled_map_[type] = false;
   }
-  inline bool is_enabled(LogType type)
+  bool is_enabled(LogType type)
   {
     return enabled_map_[type];
   }
-
-  inline void set_colorize(bool is_colorize)
+  void set_colorize(bool is_colorize)
   {
     is_colorize_ = is_colorize;
   }
@@ -73,13 +65,7 @@ public:
 private:
   Log();
   ~Log() = default;
-  std::string src_;
-  std::string filename_;
-  void log_with_location(LogType,
-                         const location&,
-                         std::ostream&,
-                         const std::string&);
-  std::string log_format_output(LogType, std::string&&);
+
   std::unordered_map<LogType, bool> enabled_map_;
   bool is_colorize_ = false;
 };
@@ -93,26 +79,36 @@ public:
   LogStream(const std::string& file,
             int line,
             LogType type,
-            std::optional<location> loc,
+            std::string&& source_location,
             std::ostream& out = std::cerr);
+  LogStream(const std::string& file,
+            int line,
+            LogType type,
+            std::string&& source_location,
+            std::vector<std::string>&& source_context,
+            std::ostream& out = std::cerr);
+
   template <typename T>
   LogStream& operator<<(const T& v)
   {
-    if (sink_.is_enabled(type_))
+    auto& sink = Log::get();
+    if (sink.is_enabled(type_))
       buf_ << v;
     return *this;
   }
   virtual ~LogStream();
 
 protected:
+  // This formats the `file_` and `line_` and may be used to prefix the message
+  // for some types of log streams.
   std::string internal_location();
 
-  Log& sink_;
+  const std::string& file_;
+  const int line_;
   LogType type_;
-  const std::optional<location> loc_;
+  std::optional<std::string> source_location_;
+  std::optional<std::vector<std::string>> source_context_;
   std::ostream& out_;
-  std::string log_file_;
-  int log_line_;
   std::ostringstream buf_;
 };
 
@@ -120,25 +116,23 @@ class LogStreamBug : public LogStream {
 public:
   LogStreamBug(const std::string& file,
                int line,
-               __attribute__((unused)) LogType,
+               __attribute__((unused)) LogType /*unused*/,
                std::ostream& out = std::cerr)
       : LogStream(file, line, LogType::BUG, out) {};
-  LogStreamBug(const std::string& file,
-               int line,
-               __attribute__((unused)) LogType,
-               std::optional<location> loc,
-               std::ostream& out = std::cerr)
-      : LogStream(file, line, LogType::BUG, loc, out) {};
-  [[noreturn]] ~LogStreamBug();
+  [[noreturn]] ~LogStreamBug() override;
 };
 
-// Usage examples:
-// 1. LOG(WARNING) << "this is a " << "warning!"; (this goes to std::cerr)
-// 2. LOG(DEBUG, std::cout) << "this is a " << " message.";
-// 3. LOG(ERROR, call.loc, std::cerr) << "this is a semantic error";
-// Note: LogType::DEBUG will prepend __FILE__ and __LINE__ to the debug message
-
 // clang-format off
+
+// Usage examples:
+//
+//   1. LOG(WARNING) << "this is a " << "warning!"; (this goes to std::cerr)
+//   2. LOG(DEBUG, std::cout) << "this is a " << " message.";
+//   3. LOG(ERROR, call.loc.source_location(), std::cerr) << "error with location";
+//   4. LOG(ERROR, call.loc.source_location(), call.loc.source_context(), std::err) << "error with context";
+//
+// Note: LogType::DEBUG will prepend __FILE__ and __LINE__ to the debug message.
+
 #define LOGSTREAM_COMMON(...) bpftrace::LogStream(__FILE__, __LINE__, __VA_ARGS__)
 #define LOGSTREAM_DEBUG(...) LOGSTREAM_COMMON(__VA_ARGS__)
 #define LOGSTREAM_V1(...) LOGSTREAM_COMMON(__VA_ARGS__)
