@@ -1,20 +1,24 @@
 #include <algorithm>
 #include <dirent.h>
 #include <fstream>
-#include <iomanip>
 #include <iostream>
-#include <regex>
 #include <string>
 #include <vector>
 
 #include "bpftrace.h"
+#include "btf.h"
 #include "cxxdemangler/cxxdemangler.h"
 #include "dwarf_parser.h"
 #include "log.h"
 #include "probe_matcher.h"
 #include "scopeguard.h"
-#include "tracefs.h"
-#include "utils.h"
+#include "tracefs/tracefs.h"
+#include "util/bpf_progs.h"
+#include "util/paths.h"
+#include "util/strings.h"
+#include "util/symbols.h"
+#include "util/system.h"
+#include "util/wildcard.h"
 
 #include <bcc/bcc_elf.h>
 #include <bcc/bcc_syms.h>
@@ -27,7 +31,7 @@ static int add_symbol(const char* symname,
                       uint64_t /*size*/,
                       void* payload)
 {
-  auto syms = static_cast<std::set<std::string>*>(payload);
+  auto* syms = static_cast<std::set<std::string>*>(payload);
   syms->insert(std::string(symname));
   return 0;
 }
@@ -40,7 +44,9 @@ std::set<std::string> ProbeMatcher::get_matches_in_stream(
     const char delim)
 {
   bool start_wildcard, end_wildcard;
-  auto tokens = get_wildcard_tokens(search_input, start_wildcard, end_wildcard);
+  auto tokens = util::get_wildcard_tokens(search_input,
+                                          start_wildcard,
+                                          end_wildcard);
 
   // Since demangled_name contains function parameters, we need to remove
   // them unless the user specified '(' in the search input (i.e. wants
@@ -49,20 +55,20 @@ std::set<std::string> ProbeMatcher::get_matches_in_stream(
   auto has_parameter = [](const std::string& token) {
     return token.find('(') != std::string::npos;
   };
-  const bool truncate_parameters = std::none_of(tokens.begin(),
-                                                tokens.end(),
-                                                has_parameter);
+  const bool truncate_parameters = std::ranges::none_of(tokens,
+
+                                                        has_parameter);
 
   std::string line;
   std::set<std::string> matches;
   while (std::getline(symbol_stream, line, delim)) {
-    if (!wildcard_match(line, tokens, start_wildcard, end_wildcard)) {
+    if (!util::wildcard_match(line, tokens, start_wildcard, end_wildcard)) {
       if (demangle_symbols) {
         auto fun_line = line;
         auto prefix = fun_line.find(':') != std::string::npos
-                          ? erase_prefix(fun_line) + ":"
+                          ? util::erase_prefix(fun_line) + ":"
                           : "";
-        if (symbol_has_cpp_mangled_signature(fun_line)) {
+        if (util::symbol_has_cpp_mangled_signature(fun_line)) {
           char* demangled_name = cxxdemangle(fun_line.c_str());
           if (!demangled_name)
             continue;
@@ -74,10 +80,10 @@ std::set<std::string> ProbeMatcher::get_matches_in_stream(
           // Match against the demanled name.
           std::string match_line = prefix + demangled_name;
           if (truncate_parameters) {
-            erase_parameter_list(match_line);
+            util::erase_parameter_list(match_line);
           }
 
-          if (wildcard_match(
+          if (util::wildcard_match(
                   match_line, tokens, start_wildcard, end_wildcard)) {
             goto out;
           }
@@ -126,9 +132,23 @@ std::set<std::string> ProbeMatcher::get_matches_for_probetype(
       symbol_stream = get_symbols_from_file(tracefs::available_events());
       break;
     }
+    // The two `has_btf_data` checks below for fentry/fexit/rawtracepoints
+    // are more about ordering than system properties.
+    // Initially, before BTF is loaded and we want to determine what modules to
+    // load BTF from we check "/sys/kernel/tracing/available_filter_functions".
+    // Then we check the BTF to filter out functions from that list that don't
+    // have any BTF definitions.
     case ProbeType::rawtracepoint: {
-      symbol_stream = get_symbols_from_file(tracefs::available_events());
-      symbol_stream = adjust_rawtracepoint(*symbol_stream);
+      symbol_stream = get_raw_tracepoint_symbols();
+      break;
+    }
+    case ProbeType::fentry:
+    case ProbeType::fexit: {
+      if (target == "bpf") {
+        symbol_stream = get_running_bpf_programs();
+      } else {
+        symbol_stream = get_fentry_symbols();
+      }
       break;
     }
     case ProbeType::usdt: {
@@ -143,25 +163,13 @@ std::set<std::string> ProbeMatcher::get_matches_for_probetype(
       symbol_stream = get_symbols_from_list(HW_PROBE_LIST);
       break;
     }
-    case ProbeType::fentry:
-    case ProbeType::fexit: {
-      // If BTF is not parsed, yet, read available_filter_functions instead.
-      // This is useful as we will use the result to extract the list of
-      // potentially used kernel modules and then only parse BTF for them.
-      if (bpftrace_->has_btf_data())
-        symbol_stream = bpftrace_->btf_->get_all_funcs();
-      else {
-        symbol_stream = get_symbols_from_traceable_funcs(true);
-      }
-      break;
-    }
     case ProbeType::iter: {
       if (!bpftrace_->has_btf_data())
         break;
 
       std::string ret;
       auto iters = bpftrace_->btf_->get_all_iters();
-      for (auto& iter : iters) {
+      for (const auto& iter : iters) {
         // second check
         if (bpftrace_->feature_->has_iter(iter))
           ret += iter + "\n";
@@ -176,12 +184,13 @@ std::set<std::string> ProbeMatcher::get_matches_for_probetype(
     case ProbeType::interval:
     case ProbeType::profile: {
       std::string ret;
-      for (auto& unit : TIME_UNITS)
+      for (const auto& unit : TIME_UNITS)
         ret += unit + ":\n";
       symbol_stream = std::make_unique<std::istringstream>(ret);
       break;
     }
     case ProbeType::special:
+    case ProbeType::benchmark:
       return { target + ":" };
     default:
       return {};
@@ -203,7 +212,7 @@ std::set<std::string> ProbeMatcher::get_matches_in_set(
   std::string stream_in;
   // Strings in the set may contain a newline character, so we use '$'
   // as a delimiter.
-  for (auto& str : set)
+  for (const auto& str : set)
     stream_in.append(str + "$");
 
   std::istringstream stream(stream_in);
@@ -227,9 +236,9 @@ std::unique_ptr<std::istream> ProbeMatcher::get_symbols_from_traceable_funcs(
     bool with_modules) const
 {
   std::string funcs;
-  for (auto& func_mod : bpftrace_->get_traceable_funcs()) {
+  for (const auto& func_mod : bpftrace_->get_traceable_funcs()) {
     if (with_modules) {
-      for (auto& mod : func_mod.second)
+      for (const auto& mod : func_mod.second)
         funcs += mod + ":" + func_mod.first + "\n";
     } else {
       funcs += func_mod.first + "\n";
@@ -238,23 +247,67 @@ std::unique_ptr<std::istream> ProbeMatcher::get_symbols_from_traceable_funcs(
   return std::make_unique<std::istringstream>(funcs);
 }
 
+std::unique_ptr<std::istream> ProbeMatcher::get_fentry_symbols() const
+{
+  if (bpftrace_->btf_->has_data() && bpftrace_->btf_->modules_loaded())
+    return bpftrace_->btf_->get_all_funcs();
+  else {
+    return get_symbols_from_traceable_funcs(true);
+  }
+}
+
+std::unique_ptr<std::istream> ProbeMatcher::get_running_bpf_programs() const
+{
+  std::string funcs;
+  auto ids_and_syms = util::get_bpf_progs();
+  for (const auto& [id, symbol] : ids_and_syms) {
+    funcs += "bpf:" + std::to_string(id) + ":" + symbol + "\n";
+  }
+  return std::make_unique<std::istringstream>(funcs);
+}
+
+std::unique_ptr<std::istream> ProbeMatcher::get_raw_tracepoint_symbols() const
+{
+  if (bpftrace_->btf_->has_data() && bpftrace_->btf_->modules_loaded()) {
+    return bpftrace_->btf_->get_all_raw_tracepoints();
+  } else {
+    std::string rts;
+    for (const auto& rt_mod : bpftrace_->get_raw_tracepoints()) {
+      for (const auto& mod : rt_mod.second)
+        rts += mod + ":" + rt_mod.first + "\n";
+    }
+    return std::make_unique<std::istringstream>(rts);
+  }
+}
+
 std::unique_ptr<std::istream> ProbeMatcher::get_func_symbols_from_file(
-    int pid,
+    std::optional<int> pid,
     const std::string& path) const
 {
   if (path.empty())
     return std::make_unique<std::istringstream>("");
 
-  std::vector<std::string> real_paths;
-  if (path == "*") {
-    if (pid > 0)
-      real_paths = get_mapped_paths_for_pid(pid);
-    else
-      real_paths = get_mapped_paths_for_running_pids();
-  } else if (path.find('*') != std::string::npos)
-    real_paths = resolve_binary_path(path, pid);
-  else
-    real_paths.push_back(path);
+  auto get_paths = [&]() -> Result<std::vector<std::string>> {
+    if (path == "*") {
+      if (pid.has_value()) {
+        return util::get_mapped_paths_for_pid(*pid);
+      } else {
+        return util::get_mapped_paths_for_running_pids();
+      }
+    } else if (path.find('*') != std::string::npos) {
+      return util::resolve_binary_path(path, pid);
+    } else {
+      return std::vector<std::string>({ path });
+    }
+  };
+  auto real_paths = get_paths();
+  if (!real_paths) {
+    // This will crash, but we need to plumb the ability to return errors
+    // through these APIs. Right now we have no other choice.
+    LOG(WARNING) << "Could not resolve binary path: " << real_paths.takeError();
+    return nullptr;
+  }
+
   struct bcc_symbol_option symbol_option;
   memset(&symbol_option, 0, sizeof(symbol_option));
   symbol_option.use_debug_file = 1;
@@ -262,38 +315,38 @@ std::unique_ptr<std::istream> ProbeMatcher::get_func_symbols_from_file(
   symbol_option.use_symbol_type = (1 << STT_FUNC) | (1 << STT_GNU_IFUNC);
 
   std::string result;
-  for (auto& real_path : real_paths) {
+  for (auto& real_path : *real_paths) {
     std::set<std::string> syms;
     // Workaround: bcc_elf_foreach_sym() can return the same symbol twice if
-    // it's also found in debug info (#1138), so a std::set is used here (and in
-    // the add_symbol callback) to ensure that each symbol will be unique in the
-    // returned string.
+    // it's also found in debug info (#1138), so a std::set is used here
+    // (and in the add_symbol callback) to ensure that each symbol will be
+    // unique in the returned string.
     int err = bcc_elf_foreach_sym(
         real_path.c_str(), add_symbol, &symbol_option, &syms);
     if (err) {
       LOG(WARNING) << "Could not list function symbols: " + real_path;
     }
-    for (auto& sym : syms)
+    for (const auto& sym : syms)
       result += real_path + ":" + sym + "\n";
   }
   return std::make_unique<std::istringstream>(result);
 }
 
 std::unique_ptr<std::istream> ProbeMatcher::get_symbols_from_usdt(
-    int pid,
+    std::optional<int> pid,
     const std::string& target) const
 {
   std::string probes;
   usdt_probe_list usdt_probes;
 
-  if (pid > 0)
-    usdt_probes = USDTHelper::probes_for_pid(pid);
+  if (pid.has_value())
+    usdt_probes = USDTHelper::probes_for_pid(*pid);
   else if (target == "*")
     usdt_probes = USDTHelper::probes_for_all_pids();
   else if (!target.empty()) {
     std::vector<std::string> real_paths;
     if (target.find('*') != std::string::npos)
-      real_paths = resolve_binary_path(target);
+      real_paths = util::resolve_binary_path(target);
     else
       real_paths.push_back(target);
 
@@ -319,7 +372,7 @@ std::unique_ptr<std::istream> ProbeMatcher::get_symbols_from_list(
     const std::vector<ProbeListItem>& probes_list) const
 {
   std::string symbols;
-  for (auto& probe : probes_list) {
+  for (const auto& probe : probes_list) {
     symbols += probe.path + ":\n";
     if (!probe.alias.empty())
       symbols += probe.alias + ":\n";
@@ -332,7 +385,7 @@ std::unique_ptr<std::istream> ProbeMatcher::get_symbols_from_list(
 std::unique_ptr<std::istream> ProbeMatcher::kernel_probe_list()
 {
   std::string probes;
-  for (auto& p : PROBE_LIST) {
+  for (const auto& p : PROBE_LIST) {
     if (!p.show_in_kernel_list) {
       continue;
     }
@@ -353,7 +406,7 @@ std::unique_ptr<std::istream> ProbeMatcher::kernel_probe_list()
 std::unique_ptr<std::istream> ProbeMatcher::userspace_probe_list()
 {
   std::string probes;
-  for (auto& p : PROBE_LIST) {
+  for (const auto& p : PROBE_LIST) {
     if (p.show_in_userspace_list) {
       probes += p.name + "\n";
     }
@@ -366,9 +419,9 @@ FuncParamLists ProbeMatcher::get_tracepoints_params(
     const std::set<std::string>& tracepoints)
 {
   FuncParamLists params;
-  for (auto& tracepoint : tracepoints) {
+  for (const auto& tracepoint : tracepoints) {
     auto event = tracepoint;
-    auto category = erase_prefix(event);
+    auto category = util::erase_prefix(event);
 
     std::string format_file_path = tracefs::event_format_file(category, event);
     std::ifstream format_file(format_file_path.c_str());
@@ -382,10 +435,10 @@ FuncParamLists ProbeMatcher::get_tracepoints_params(
     // Skip lines until the first empty line
     do {
       getline(format_file, line);
-    } while (line.length() > 0);
+    } while (!line.empty());
 
     while (getline(format_file, line)) {
-      if (line.find("\tfield:") == 0) {
+      if (line.starts_with("\tfield:")) {
         size_t col_pos = line.find(':') + 1;
         params[tracepoint].push_back(
             line.substr(col_pos, line.find(';') - col_pos));
@@ -402,7 +455,7 @@ FuncParamLists ProbeMatcher::get_iters_params(
   FuncParamLists params;
   std::set<std::string> funcs;
 
-  for (auto& iter : iters)
+  for (const auto& iter : iters)
     funcs.insert(prefix + iter);
 
   params = bpftrace_->btf_->get_params(funcs);
@@ -425,9 +478,9 @@ FuncParamLists ProbeMatcher::get_uprobe_params(
   FuncParamLists params;
   static std::set<std::string> warned_paths;
 
-  for (auto& match : uprobes) {
+  for (const auto& match : uprobes) {
     std::string fun = match;
-    std::string path = erase_prefix(fun);
+    std::string path = util::erase_prefix(fun);
     auto dwarf = Dwarf::GetFromBinary(nullptr, path);
     if (dwarf)
       params.emplace(match, dwarf->get_function_params(fun));
@@ -454,25 +507,31 @@ void ProbeMatcher::list_probes(ast::Program* prog)
         else if (probe_type == ProbeType::fentry ||
                  probe_type == ProbeType::fexit)
           param_lists = bpftrace_->btf_->get_params(matches);
+        else if (probe_type == ProbeType::rawtracepoint)
+          param_lists = bpftrace_->btf_->get_rawtracepoint_params(matches);
         else if (probe_type == ProbeType::iter)
           param_lists = get_iters_params(matches);
         else if (probe_type == ProbeType::uprobe)
           param_lists = get_uprobe_params(matches);
+        else if (probe_type == ProbeType::kprobe)
+          param_lists = bpftrace_->btf_->get_kprobes_params(matches);
+        else if (probe_type == ProbeType::kretprobe)
+          param_lists = bpftrace_->btf_->get_kretprobes_params(matches);
       }
 
-      for (auto& match : matches) {
+      for (const auto& match : matches) {
         std::string match_print = match;
         if (ap->lang == "cpp") {
-          std::string target = erase_prefix(match_print);
+          std::string target = util::erase_prefix(match_print);
           char* demangled_name = cxxdemangle(match_print.c_str());
           SCOPE_EXIT
           {
             ::free(demangled_name);
           };
 
-          // demangled name may contain symbols not accepted by the attach point
-          // parser, so surround it with quotes to make the entry directly
-          // usable as an attach point
+          // demangled name may contain symbols not accepted by the attach
+          // point parser, so surround it with quotes to make the entry
+          // directly usable as an attach point
           auto func = demangled_name ? "\"" + std::string(demangled_name) + "\""
                                      : match_print;
 
@@ -502,21 +561,36 @@ std::set<std::string> ProbeMatcher::get_matches_for_ap(
         search_input = attach_point.func;
       break;
     }
-    case ProbeType::iter:
     case ProbeType::rawtracepoint: {
+      search_input = attach_point.target + ":" + attach_point.func;
+      break;
+    }
+    case ProbeType::iter: {
       search_input = attach_point.func;
       break;
     }
+    case ProbeType::fentry:
+    case ProbeType::fexit: {
+      if (attach_point.target == "bpf") {
+        search_input = "bpf:" +
+                       (attach_point.bpf_prog_id
+                            ? std::to_string(attach_point.bpf_prog_id)
+                            : "*") +
+                       ":" + attach_point.func;
+        break;
+      }
+      [[fallthrough]];
+    }
     case ProbeType::special:
+    case ProbeType::benchmark:
     case ProbeType::uprobe:
     case ProbeType::uretprobe:
     case ProbeType::watchpoint:
     case ProbeType::asyncwatchpoint:
-    case ProbeType::tracepoint:
-    case ProbeType::fentry:
-    case ProbeType::fexit: {
-      // Do not expand "target:" as that would match all functions in target.
-      // This may occur when an absolute address is given instead of a function.
+    case ProbeType::tracepoint: {
+      // Do not expand "target:" as that would match all functions in
+      // target. This may occur when an absolute address is given instead of
+      // a function.
       if (attach_point.func.empty())
         return { attach_point.target + ":" };
 
@@ -533,11 +607,11 @@ std::set<std::string> ProbeMatcher::get_matches_for_ap(
     case ProbeType::usdt: {
       auto target = attach_point.target;
       // If PID is specified, targets in symbol_stream will have the
-      // "/proc/<PID>/root" prefix followed by an absolute path, so we make the
-      // target absolute and add a leading wildcard.
-      if (bpftrace_->pid() > 0) {
+      // "/proc/<PID>/root" prefix followed by an absolute path, so we make
+      // the target absolute and add a leading wildcard.
+      if (bpftrace_->pid().has_value()) {
         if (!target.empty()) {
-          if (auto abs_target = abs_path(target))
+          if (auto abs_target = util::abs_path(target))
             target = "*" + abs_target.value();
         } else
           target = "*";
@@ -561,7 +635,7 @@ std::set<std::string> ProbeMatcher::get_matches_for_ap(
 std::set<std::string> ProbeMatcher::expand_probetype_kernel(
     const std::string& probe_type)
 {
-  if (has_wildcard(probe_type))
+  if (util::has_wildcard(probe_type))
     return get_matches_in_stream(probe_type, *kernel_probe_list());
   else
     return { probe_type };
@@ -570,7 +644,7 @@ std::set<std::string> ProbeMatcher::expand_probetype_kernel(
 std::set<std::string> ProbeMatcher::expand_probetype_userspace(
     const std::string& probe_type)
 {
-  if (has_wildcard(probe_type))
+  if (util::has_wildcard(probe_type))
     return get_matches_in_stream(probe_type, *userspace_probe_list());
   else
     return { probe_type };
@@ -585,23 +659,7 @@ void ProbeMatcher::list_structs(const std::string& search)
   if (bt_verbose)
     search_input += " *{*}*";
 
-  for (auto& match : get_matches_in_set(search_input, structs))
+  for (const auto& match : get_matches_in_set(search_input, structs))
     std::cout << match << std::endl;
 }
-
-std::unique_ptr<std::istream> ProbeMatcher::adjust_rawtracepoint(
-    std::istream& symbol_list) const
-{
-  auto new_list = std::make_unique<std::stringstream>();
-  std::string line;
-  while (std::getline(symbol_list, line, '\n')) {
-    if ((line.find("syscalls:sys_enter_") != std::string::npos) ||
-        (line.find("syscalls:sys_exit_") != std::string::npos))
-      continue;
-    erase_prefix(line);
-    *new_list << line << "\n";
-  }
-  return new_list;
-}
-
 } // namespace bpftrace
