@@ -9,60 +9,123 @@
 #include <vector>
 
 #include <cereal/access.hpp>
+#include <cereal/types/variant.hpp>
 
+#include "ast/location.h"
 #include "format_string.h"
-#include "location.hh"
+#include "globalvars.h"
+#include "map_info.h"
 #include "struct.h"
 #include "types.h"
+#include "util/bpf_funcs.h"
 
 namespace bpftrace {
 
 class BPFtrace;
 
-struct HelperErrorInfo {
-  int func_id = -1;
-  location loc;
+static const auto DIVIDE_BY_ZERO_MSG =
+    "Divide or modulo by 0 detected. This can lead to unexpected "
+    "results. 1 is being used as the result.";
+
+static const auto ARRAY_ACCESS_OOB_MSG =
+    "Array access out of bounds. This can lead to unexpected "
+    "results.";
+
+enum class RuntimeErrorId {
+  DIVIDE_BY_ZERO,
+  HELPER_ERROR,
+  ARRAY_ACCESS_OOB,
 };
 
-struct LinearHistogramArgs {
-  long min = -1;
-  long max = -1;
-  long step = -1;
+enum class PrintfSeverity {
+  NONE,
+  ERROR,
+};
 
-  bool operator==(const LinearHistogramArgs &other)
-  {
-    return min == other.min && max == other.max && step == other.step;
-  }
-  bool operator!=(const LinearHistogramArgs &other)
-  {
-    return !(*this == other);
-  }
+struct SourceLocation {
+  std::string filename;
+  int line;
+  int column;
+  std::string source_location;
+  std::vector<std::string> source_context;
 
 private:
   friend class cereal::access;
   template <typename Archive>
   void serialize(Archive &archive)
   {
-    archive(min, max, step);
+    archive(filename, line, column, source_location, source_context);
   }
 };
 
-struct MapInfo {
-  SizedType key_type;
-  SizedType value_type;
-  std::optional<LinearHistogramArgs> lhist_args;
-  std::optional<int> hist_bits_arg;
-  int id = -1;
+class SourceInfo {
+public:
+  SourceInfo(const ast::Location &loc)
+  {
+    auto curr_loc = loc;
+
+    while (curr_loc) {
+      locations.emplace_back(curr_loc->filename(),
+                             curr_loc->line(),
+                             curr_loc->column(),
+                             curr_loc->source_location(),
+                             curr_loc->source_context());
+      auto &parent = curr_loc->parent;
+      if (parent) {
+        curr_loc = parent->loc;
+      } else {
+        break;
+      }
+    }
+  }
+
+  SourceInfo() = default;
+
+  std::vector<SourceLocation> locations;
 
 private:
   friend class cereal::access;
   template <typename Archive>
   void serialize(Archive &archive)
   {
-    archive(key_type, value_type, lhist_args, hist_bits_arg, id);
+    archive(locations);
   }
 };
 
+class RuntimeErrorInfo : public SourceInfo {
+public:
+  // This class effectively wraps a location, but preserves only the parts that
+  // are needed to emit the error in a useful way. This is because it may be
+  // serialized and used by a separate runtime.
+  RuntimeErrorInfo(RuntimeErrorId error_id,
+                   libbpf::bpf_func_id func_id,
+                   const ast::Location &loc)
+      : SourceInfo(loc), error_id(error_id), func_id(func_id)
+  {
+  }
+
+  RuntimeErrorInfo(RuntimeErrorId error_id, const ast::Location &loc)
+      : RuntimeErrorInfo(error_id, static_cast<libbpf::bpf_func_id>(-1), loc) {
+        };
+
+  RuntimeErrorInfo()
+      : error_id(RuntimeErrorId::HELPER_ERROR),
+        func_id(static_cast<libbpf::bpf_func_id>(-1)) {};
+
+  RuntimeErrorId error_id;
+  libbpf::bpf_func_id func_id;
+
+private:
+  friend class cereal::access;
+  template <typename Archive>
+  void serialize(Archive &archive)
+  {
+    archive(error_id, func_id);
+  }
+};
+
+std::ostream &operator<<(std::ostream &os, const RuntimeErrorInfo &info);
+
 // This class contains script-specific metadata that bpftrace's runtime needs.
 //
 // This class is intended to completely encapsulate all of a script's runtime
@@ -85,7 +148,9 @@ public:
   void load_state(const uint8_t *ptr, size_t len);
 
   // Async argument metadata
-  std::vector<std::tuple<FormatString, std::vector<Field>>> printf_args;
+  std::vector<
+      std::tuple<FormatString, std::vector<Field>, PrintfSeverity, SourceInfo>>
+      printf_args;
   std::vector<std::tuple<FormatString, std::vector<Field>>> system_args;
   // fmt strings for BPF helpers (bpf_seq_printf, bpf_trace_printk)
   std::vector<FormatString> bpf_print_fmts;
@@ -127,13 +192,13 @@ public:
   // pass should be collecting this, but it's complex to move the logic.
   //
   // Don't add more async arguments here!.
-  std::unordered_map<int64_t, struct HelperErrorInfo> helper_error_info;
+  std::unordered_map<int64_t, RuntimeErrorInfo> runtime_error_info;
   std::vector<std::string> probe_ids;
 
   // Map metadata
   std::map<std::string, MapInfo> maps_info;
-  std::unordered_set<bpftrace::globalvars::GlobalVar> needed_global_vars;
-  bool needs_perf_event_map = false;
+  globalvars::GlobalVars global_vars;
+  bool using_skboutput = false;
 
   // Probe metadata
   //
@@ -141,6 +206,8 @@ public:
   // be collecting this, but it's complex to move the logic.
   std::vector<Probe> probes;
   std::unordered_map<std::string, Probe> special_probes;
+  std::vector<Probe> benchmark_probes;
+  std::vector<Probe> signal_probes;
   std::vector<Probe> watchpoint_probes;
 
   // List of probes using userspace symbol resolution
@@ -158,15 +225,16 @@ private:
             strftime_args,
             cat_args,
             non_map_print_args,
-            // Hard to annotate flex types, so skip
-            // helper_error_info,
+            runtime_error_info,
             printf_args,
             probe_ids,
             maps_info,
-            needed_global_vars,
-            needs_perf_event_map,
+            global_vars,
+            using_skboutput,
             probes,
-            special_probes);
+            signal_probes,
+            special_probes,
+            benchmark_probes);
   }
 };
 
