@@ -4,15 +4,11 @@
 using ::testing::_;
 using ::testing::Return;
 
-namespace bpftrace {
-namespace test {
-namespace codegen {
+namespace bpftrace::test::codegen {
 
 TEST(codegen, args_multiple_tracepoints_wild)
 {
   test("tracepoint:sched:sched_* { @[args.common_field] = 1; }", NAME);
 }
 
-} // namespace codegen
-} // namespace test
-} // namespace bpftrace
+} // namespace bpftrace::test::codegen
