@@ -1,10 +1,6 @@
 #include "common.h"
 
-namespace bpftrace {
-namespace test {
-namespace codegen {
-
-namespace array_integer_equal_comparison {
+namespace bpftrace::test::codegen::array_integer_equal_comparison {
 constexpr auto PROG = "struct Foo { int arr[4]; }"
                       "kprobe:f"
                       "{"
@@ -21,8 +17,4 @@ TEST(codegen, array_integer_equal_comparison)
   test(PROG, NAME);
 }
 
-} // namespace array_integer_equal_comparison
-
-} // namespace codegen
-} // namespace test
-} // namespace bpftrace
+} // namespace bpftrace::test::codegen::array_integer_equal_comparison
