@@ -1,14 +1,10 @@
 #include "common.h"
 
-namespace bpftrace {
-namespace test {
-namespace codegen {
+namespace bpftrace::test::codegen {
 
 TEST(codegen, basic_while_loop)
 {
   test("i:s:1 { $a = 1; while ($a <= 150) { @=$a++; }}", NAME);
 }
 
-} // namespace codegen
-} // namespace test
-} // namespace bpftrace
+} // namespace bpftrace::test::codegen
