@@ -1,8 +1,6 @@
 #include "common.h"
 
-namespace bpftrace {
-namespace test {
-namespace codegen {
+namespace bpftrace::test::codegen {
 
 TEST(codegen, builtin_cpid)
 {
@@ -13,6 +11,4 @@ TEST(codegen, builtin_cpid)
   test(*bpftrace, "kprobe:f { @ = cpid }", NAME);
 }
 
-} // namespace codegen
-} // namespace test
-} // namespace bpftrace
+} // namespace bpftrace::test::codegen
