@@ -1,12 +1,10 @@
 #include "common.h"
 
-namespace bpftrace {
-namespace test {
-namespace codegen {
+namespace bpftrace::test::codegen {
 
 TEST(codegen, builtin_ctx_field)
 {
-  std::string prog = R"END(
+  std::string prog = R"(
 struct c {
   char c;
 };
@@ -27,11 +25,9 @@ kprobe:f {
   @d = $x->d->c;
   @e = $x->e;
 }
-)END";
+)";
 
   test(prog, NAME);
 }
 
-} // namespace codegen
-} // namespace test
-} // namespace bpftrace
+} // namespace bpftrace::test::codegen
