@@ -1,8 +1,6 @@
 #include "common.h"
 
-namespace bpftrace {
-namespace test {
-namespace codegen {
+namespace bpftrace::test::codegen {
 
 TEST(codegen, builtin_func_kprobe)
 {
@@ -29,6 +27,4 @@ TEST(codegen, builtin_func_fentry)
   test("fentry:f { @x = func }", NAME);
 }
 
-} // namespace codegen
-} // namespace test
-} // namespace bpftrace
+} // namespace bpftrace::test::codegen
