@@ -1,9 +1,7 @@
 #include "../mocks.h"
 #include "common.h"
 
-namespace bpftrace {
-namespace test {
-namespace codegen {
+namespace bpftrace::test::codegen {
 
 TEST(codegen, builtin_probe)
 {
@@ -17,6 +15,4 @@ TEST(codegen, builtin_probe_comparison)
       NAME);
 }
 
-} // namespace codegen
-} // namespace test
-} // namespace bpftrace
+} // namespace bpftrace::test::codegen
