@@ -1,9 +1,7 @@
 #include "../mocks.h"
 #include "common.h"
 
-namespace bpftrace {
-namespace test {
-namespace codegen {
+namespace bpftrace::test::codegen {
 
 using ::testing::Return;
 
@@ -12,6 +10,4 @@ TEST(codegen, builtin_probe_wild)
   test("tracepoint:sched:sched_on* { @x = probe }", NAME);
 }
 
-} // namespace codegen
-} // namespace test
-} // namespace bpftrace
+} // namespace bpftrace::test::codegen
