@@ -1,8 +1,6 @@
 #include "common.h"
 
-namespace bpftrace {
-namespace test {
-namespace codegen {
+namespace bpftrace::test::codegen {
 
 TEST(codegen, call_uptr)
 {
@@ -10,6 +8,4 @@ TEST(codegen, call_uptr)
   test("k:f { @=*uptr((int32*) arg0 ); }", std::string(NAME) + "_2");
 }
 
-} // namespace codegen
-} // namespace test
-} // namespace bpftrace
+} // namespace bpftrace::test::codegen
