@@ -4,7 +4,7 @@ namespace bpftrace::test::codegen {
 
 TEST(codegen, for_map_strings)
 {
-  test(R"(BEGIN { @map["abc"] = "xyz"; for ($kv : @map) { @x = $kv; } })",
+  test(R"(begin { @map["abc"] = "xyz"; for ($kv : @map) { @x = $kv; } })",
        NAME);
 }
 
