@@ -1,8 +1,6 @@
 #include "common.h"
 
-namespace bpftrace {
-namespace test {
-namespace codegen {
+namespace bpftrace::test::codegen {
 
 #ifdef __x86_64__
 TEST(codegen, intptrcast_assign_var)
@@ -11,6 +9,4 @@ TEST(codegen, intptrcast_assign_var)
 }
 #endif
 
-} // namespace codegen
-} // namespace test
-} // namespace bpftrace
+} // namespace bpftrace::test::codegen
