@@ -1,8 +1,6 @@
 #include "common.h"
 
-namespace bpftrace {
-namespace test {
-namespace codegen {
+namespace bpftrace::test::codegen {
 
 #ifdef __x86_64__
 TEST(codegen, intptrcast_call)
@@ -12,6 +10,4 @@ TEST(codegen, intptrcast_call)
 }
 #endif
 
-} // namespace codegen
-} // namespace test
-} // namespace bpftrace
+} // namespace bpftrace::test::codegen
