@@ -1,13 +1,11 @@
 #include "common.h"
 
-namespace bpftrace {
-namespace test {
-namespace codegen {
+namespace bpftrace::test::codegen {
 
 TEST(codegen, logical_and_or_different_type)
 {
   test("struct Foo { int m; }"
-       "BEGIN"
+       "begin"
        "{"
        "  $foo = *(struct Foo*)0;"
        "  printf(\"%d %d %d %d\", $foo.m && 0, 1 && $foo.m, $foo.m || 0, 0 || "
@@ -16,6 +14,4 @@ TEST(codegen, logical_and_or_different_type)
        NAME);
 }
 
-} // namespace codegen
-} // namespace test
-} // namespace bpftrace
+} // namespace bpftrace::test::codegen
