@@ -1,9 +1,7 @@
 #include "../mocks.h"
 #include "common.h"
 
-namespace bpftrace {
-namespace test {
-namespace codegen {
+namespace bpftrace::test::codegen {
 
 TEST(codegen, map_key_probe)
 {
@@ -13,6 +11,4 @@ TEST(codegen, map_key_probe)
        NAME);
 }
 
-} // namespace codegen
-} // namespace test
-} // namespace bpftrace
+} // namespace bpftrace::test::codegen
