@@ -1,8 +1,6 @@
 #include "common.h"
 
-namespace bpftrace {
-namespace test {
-namespace codegen {
+namespace bpftrace::test::codegen {
 
 TEST(codegen, count_cast)
 {
@@ -56,7 +54,7 @@ TEST(codegen, avg_cast_loop)
 
 TEST(codegen, count_no_cast_for_print)
 {
-  test("BEGIN { @ = count(); print(@) }", NAME);
+  test("begin { @ = count(); print(@) }", NAME);
 }
 
 TEST(codegen, count_cast_loop_multi_key)
@@ -72,6 +70,4 @@ TEST(codegen, count_cast_loop_stack_key)
        NAME);
 }
 
-} // namespace codegen
-} // namespace test
-} // namespace bpftrace
+} // namespace bpftrace::test::codegen
