@@ -1,8 +1,6 @@
 #include "common.h"
 
-namespace bpftrace {
-namespace test {
-namespace codegen {
+namespace bpftrace::test::codegen {
 
 TEST(codegen, pointer_if_condition)
 {
@@ -24,6 +22,4 @@ TEST(codegen, pointer_logical_or)
   test("kprobe:f { $v = (int16*)1; if ($v || 0) {} }", NAME);
 }
 
-} // namespace codegen
-} // namespace test
-} // namespace bpftrace
+} // namespace bpftrace::test::codegen
