@@ -1,15 +1,13 @@
 #include "common.h"
 
-namespace bpftrace {
-namespace test {
-namespace codegen {
+namespace bpftrace::test::codegen {
 
 TEST(codegen, struct_nested_struct_anon)
 {
   test("struct Foo { struct { int x; } bar; }"
        "kprobe:f"
        "{"
-       "  $foo = (struct Foo)0;"
+       "  $foo = *(struct Foo*)arg0;"
        "  @x = $foo.bar.x;"
        "}",
        std::string(NAME) + "_1");
@@ -17,12 +15,10 @@ TEST(codegen, struct_nested_struct_anon)
   test("struct Foo { struct { int x; } bar; }"
        "kprobe:f"
        "{"
-       "  $foo = (struct Foo*)0;"
+       "  $foo = (struct Foo*)arg0;"
        "  @x = $foo->bar.x;"
        "}",
        std::string(NAME) + "_2");
 }
 
-} // namespace codegen
-} // namespace test
-} // namespace bpftrace
+} // namespace bpftrace::test::codegen
