@@ -1,8 +1,6 @@
 #include "common.h"
 
-namespace bpftrace {
-namespace test {
-namespace codegen {
+namespace bpftrace::test::codegen {
 
 // Trip counts under 10 are usually unrolled automatically.
 //
@@ -12,6 +10,4 @@ TEST(codegen, while_loop_no_unroll)
   test("i:s:1 { $a = 0; while ($a <= 10) { @=$a++; }}", NAME);
 }
 
-} // namespace codegen
-} // namespace test
-} // namespace bpftrace
+} // namespace bpftrace::test::codegen
