@@ -1,4 +1,5 @@
 #include "ast/passes/collect_nodes.h"
+#include "ast/context.h"
 #include "gtest/gtest.h"
 
 #include <functional>
@@ -23,7 +24,7 @@ TEST(CollectNodes, direct)
   ASTContext ctx;
   auto &var = *ctx.make_node<Variable>("myvar", bpftrace::location{});
 
-  CollectNodes<Variable> visitor(ctx);
+  CollectNodes<Variable> visitor;
   visitor.visit(var);
 
   test({ var }, visitor.nodes());
@@ -34,9 +35,9 @@ TEST(CollectNodes, indirect)
   ASTContext ctx;
   auto &var = *ctx.make_node<Variable>("myvar", bpftrace::location{});
   auto &unop = *ctx.make_node<Unop>(
-      Operator::INCREMENT, &var, false, bpftrace::location{});
+      &var, Operator::INCREMENT, false, bpftrace::location{});
 
-  CollectNodes<Variable> visitor(ctx);
+  CollectNodes<Variable> visitor;
   visitor.visit(unop);
 
   test({ var }, visitor.nodes());
@@ -47,9 +48,9 @@ TEST(CollectNodes, none)
   ASTContext ctx;
   auto &map = *ctx.make_node<Map>("myvar", bpftrace::location{});
   auto &unop = *ctx.make_node<Unop>(
-      Operator::INCREMENT, &map, false, bpftrace::location{});
+      &map, Operator::INCREMENT, false, bpftrace::location{});
 
-  CollectNodes<Variable> visitor(ctx);
+  CollectNodes<Variable> visitor;
   visitor.visit(unop);
 
   test({}, visitor.nodes());
@@ -60,13 +61,13 @@ TEST(CollectNodes, multiple_runs)
   ASTContext ctx;
   auto &var1 = *ctx.make_node<Variable>("myvar1", bpftrace::location{});
   auto &unop1 = *ctx.make_node<Unop>(
-      Operator::INCREMENT, &var1, false, bpftrace::location{});
+      &var1, Operator::INCREMENT, false, bpftrace::location{});
 
   auto &var2 = *ctx.make_node<Variable>("myvar2", bpftrace::location{});
   auto &unop2 = *ctx.make_node<Unop>(
-      Operator::INCREMENT, &var2, false, bpftrace::location{});
+      &var2, Operator::INCREMENT, false, bpftrace::location{});
 
-  CollectNodes<Variable> visitor(ctx);
+  CollectNodes<Variable> visitor;
   visitor.visit(unop1);
   visitor.visit(unop2);
 
@@ -81,7 +82,7 @@ TEST(CollectNodes, multiple_children)
   auto &binop = *ctx.make_node<Binop>(
       &var1, Operator::PLUS, &var2, bpftrace::location{});
 
-  CollectNodes<Variable> visitor(ctx);
+  CollectNodes<Variable> visitor;
   visitor.visit(binop);
 
   test({ var1, var2 }, visitor.nodes());
@@ -95,7 +96,7 @@ TEST(CollectNodes, predicate)
   auto &binop = *ctx.make_node<Binop>(
       &var1, Operator::PLUS, &var2, bpftrace::location{});
 
-  CollectNodes<Variable> visitor(ctx);
+  CollectNodes<Variable> visitor;
   visitor.visit(binop, [](const auto &var) { return var.ident == "myvar2"; });
 
   test({ var2 }, visitor.nodes());
@@ -112,7 +113,7 @@ TEST(CollectNodes, nested)
   auto &binop2 = *ctx.make_node<Binop>(
       &binop1, Operator::MINUS, &var3, bpftrace::location{});
 
-  CollectNodes<Binop> visitor(ctx);
+  CollectNodes<Binop> visitor;
   visitor.visit(binop2,
                 [](const auto &binop) { return binop.op == Operator::PLUS; });
 
