@@ -363,13 +363,13 @@ TEST(fold_literals, binary)
   test("1 << 1", "int: 2 :: [int64]");
   test("1 << 2", "int: 4 :: [int64]");
   test("1 << 63", "int: 9223372036854775808 :: [uint64]");
-  test("1 << 64", "int: 1 :: [int64]"); // Wraps around, still signed
   test("0xff << 8", "int: 65280 :: [int64]");
   test("0xff << 56", "int: 18374686479671623680 :: [uint64]");
   test("-1 << 1", "negative int: -2");
   test("-1 << 63", "negative int: -9223372036854775808");
   test("0x7fffffffffffffff << 1", "int: 18446744073709551614 :: [uint64]");
   test("0x8000000000000000 << 1", "int: 0 :: [uint64]"); // Legal overflow
+  test_error("1 << 64", "overflow");
 
   test("8 >> 1", "int: 4 :: [int64]");
   test("8 >> 2", "int: 2 :: [int64]");
@@ -381,6 +381,7 @@ TEST(fold_literals, binary)
   test("-8 >> 2", "negative int: -2"); // Sign extension
   test("0x8000000000000000 >> 1", "int: 4611686018427387904 :: [uint64]");
   test("0x8000000000000000 >> 63", "int: 1 :: [uint64]");
+  test_error("1 >> 64", "overflow");
 
   test("true & true", "bool: true");
   test("true & false", "bool: false");
