@@ -1,5 +1,6 @@
 #include "mocks.h"
-#include "tracefs.h"
+#include "tracefs/tracefs.h"
+#include "gmock/gmock-nice-strict.h"
 
 namespace bpftrace::test {
 
@@ -23,7 +24,8 @@ void setup_mock_probe_matcher(MockProbeMatcher &matcher)
   ON_CALL(matcher, get_symbols_from_traceable_funcs(true)).WillByDefault([]() {
     std::string ksyms = "kernel_mod:func_in_mod\n"
                         "kernel_mod:other_func_in_mod\n"
-                        "other_kernel_mod:func_in_mod\n";
+                        "other_kernel_mod:func_in_mod\n"
+                        "vmlinux:queued_spin_lock_slowpath\n";
     auto myval = std::unique_ptr<std::istream>(new std::istringstream(ksyms));
     return myval;
   });
@@ -37,11 +39,27 @@ void setup_mock_probe_matcher(MockProbeMatcher &matcher)
                                   "notsched:bar\n"
                                   "file:filename\n"
                                   "tcp:some_tcp_tp\n"
-                                  "btf:tag\n";
+                                  "btf:tag\n"
+                                  "vmlinux:event_rt\n";
         return std::unique_ptr<std::istream>(
             new std::istringstream(tracepoints));
       });
 
+  ON_CALL(matcher, get_raw_tracepoint_symbols()).WillByDefault([]() {
+    std::string rawtracepoints = "vmlinux:event_rt\n"
+                                 "vmlinux:sched_switch\n";
+    return std::unique_ptr<std::istream>(
+        new std::istringstream(rawtracepoints));
+  });
+
+  ON_CALL(matcher, get_fentry_symbols()).WillByDefault([]() {
+    std::string funcs = "vmlinux:func_1\n"
+                        "vmlinux:func_2\n"
+                        "vmlinux:func_3\n"
+                        "vmlinux:queued_spin_lock_slowpath\n";
+    return std::unique_ptr<std::istream>(new std::istringstream(funcs));
+  });
+
   std::string sh_usyms = "/bin/sh:first_open\n"
                          "/bin/sh:second_open\n"
                          "/bin/sh:open_as_well\n"
@@ -52,15 +70,16 @@ void setup_mock_probe_matcher(MockProbeMatcher &matcher)
                          "/bin/sh:_Z18cpp_mangled_suffixv\n";
   std::string bash_usyms = "/bin/bash:first_open\n";
   ON_CALL(matcher, get_func_symbols_from_file(_, "/bin/sh"))
-      .WillByDefault([sh_usyms](int, const std::string &) {
+      .WillByDefault([sh_usyms](std::optional<int>, const std::string &) {
         return std::unique_ptr<std::istream>(new std::istringstream(sh_usyms));
       });
 
   ON_CALL(matcher, get_func_symbols_from_file(_, "/bin/*sh"))
-      .WillByDefault([sh_usyms, bash_usyms](int, const std::string &) {
-        return std::unique_ptr<std::istream>(
-            new std::istringstream(sh_usyms + bash_usyms));
-      });
+      .WillByDefault(
+          [sh_usyms, bash_usyms](std::optional<int>, const std::string &) {
+            return std::unique_ptr<std::istream>(
+                new std::istringstream(sh_usyms + bash_usyms));
+          });
 
   std::string sh_usdts = "/bin/sh:prov1:tp1\n"
                          "/bin/sh:prov1:tp2\n"
@@ -69,21 +88,27 @@ void setup_mock_probe_matcher(MockProbeMatcher &matcher)
                          "/bin/sh:nahprov:tp\n";
   std::string bash_usdts = "/bin/bash:prov1:tp3\n";
   ON_CALL(matcher, get_symbols_from_usdt(_, "/bin/sh"))
-      .WillByDefault([sh_usdts](int, const std::string &) {
+      .WillByDefault([sh_usdts](std::optional<int>, const std::string &) {
         return std::unique_ptr<std::istream>(new std::istringstream(sh_usdts));
       });
   ON_CALL(matcher, get_symbols_from_usdt(_, "/bin/*sh"))
-      .WillByDefault([sh_usdts, bash_usdts](int, const std::string &) {
-        return std::unique_ptr<std::istream>(
-            new std::istringstream(sh_usdts + bash_usdts));
-      });
+      .WillByDefault(
+          [sh_usdts, bash_usdts](std::optional<int>, const std::string &) {
+            return std::unique_ptr<std::istream>(
+                new std::istringstream(sh_usdts + bash_usdts));
+          });
+
+  ON_CALL(matcher, get_running_bpf_programs()).WillByDefault([]() {
+    std::string bpf_progs = "bpf:123:func_1\n"
+                            "bpf:123:func_2\n"
+                            "bpf:456:func_1\n";
+    return std::unique_ptr<std::istream>(new std::istringstream(bpf_progs));
+  });
 }
 
 void setup_mock_bpftrace(MockBPFtrace &bpftrace)
 {
   bpftrace.delta_taitime_ = timespec{};
-
-  bpftrace.parse_btf({ "vmlinux" });
   // Fill in some default tracepoint struct definitions
   bpftrace.structs.Add("struct _tracepoint_sched_sched_one", 8);
   bpftrace.structs.Lookup("struct _tracepoint_sched_sched_one")
@@ -174,7 +199,7 @@ std::unique_ptr<MockUSDTHelper> get_mock_usdt_helper(int num_locations)
   auto usdt_helper = std::make_unique<NiceMock<MockUSDTHelper>>();
 
   ON_CALL(*usdt_helper, find(_, _, _, _))
-      .WillByDefault([num_locations](int,
+      .WillByDefault([num_locations](std::optional<int>,
                                      const std::string &,
                                      const std::string &,
                                      const std::string &) {
