@@ -1,41 +1,46 @@
-#include "gmock/gmock.h"
-#include "gtest/gtest.h"
-
-#include "ast/passes/field_analyser.h"
 #include "ast/passes/portability_analyser.h"
+#include "ast/attachpoint_parser.h"
+#include "ast/passes/field_analyser.h"
+#include "ast/passes/map_sugar.h"
+#include "ast/passes/named_param.h"
+#include "ast/passes/probe_expansion.h"
 #include "ast/passes/semantic_analyser.h"
-#include "clang_parser.h"
+#include "ast/passes/type_system.h"
+#include "btf_common.h"
 #include "driver.h"
 #include "mocks.h"
+#include "gtest/gtest.h"
 
 namespace bpftrace::test::portability_analyser {
 
-#include "btf_common.h"
-
 using ::testing::_;
 
 void test(BPFtrace &bpftrace, const std::string &input, int expected_result = 0)
 {
-  Driver driver(bpftrace);
-  std::stringstream out;
+  ast::ASTContext ast("stdin", input);
   std::stringstream msg;
   msg << "\nInput:\n" << input << "\n\nOutput:\n";
 
-  ASSERT_EQ(driver.parse_str(input), 0);
+  ast::CDefinitions no_c_defs; // Output from clang parser.
+  ast::TypeMetadata no_types;  // No external types defined.
 
-  ast::FieldAnalyser fields(driver.ctx, bpftrace, out);
-  ASSERT_EQ(fields.analyse(), 0) << msg.str() << out.str();
-
-  ClangParser clang;
-  ASSERT_TRUE(clang.parse(driver.ctx.root, bpftrace));
-
-  ASSERT_EQ(driver.parse_str(input), 0);
-  out.str("");
-  ast::SemanticAnalyser semantics(driver.ctx, bpftrace, out, false);
-  ASSERT_EQ(semantics.analyse(), 0) << msg.str() << out.str();
-
-  ast::PortabilityAnalyser portability(driver.ctx, out);
-  EXPECT_EQ(portability.analyse(), expected_result) << msg.str() << out.str();
+  // N.B. No macro expansion.
+  auto ok = ast::PassManager()
+                .put(ast)
+                .put(bpftrace)
+                .put(no_c_defs)
+                .put(no_types)
+                .add(CreateParsePass())
+                .add(ast::CreateParseAttachpointsPass())
+                .add(ast::CreateProbeExpansionPass())
+                .add(ast::CreateMapSugarPass())
+                .add(ast::CreateFieldAnalyserPass())
+                .add(ast::CreateNamedParamsPass())
+                .add(ast::CreateSemanticPass())
+                .add(ast::CreatePortabilityPass())
+                .run();
+  ASSERT_TRUE(bool(ok));
+  EXPECT_EQ(int(!ast.diagnostics().ok()), expected_result) << msg.str();
 }
 
 void test(const std::string &input, int expected_result = 0)
@@ -46,7 +51,7 @@ void test(const std::string &input, int expected_result = 0)
 
 TEST(portability_analyser, generic_field_access_disabled)
 {
-  test("struct Foo { int x;} BEGIN { $f = (struct Foo *)0; $f->x; }", 1);
+  test("struct Foo { int x;} begin { $f = (struct Foo *)0; $f->x; }", 1);
 }
 
 TEST(portability_analyser, tracepoint_field_access)
@@ -76,20 +81,20 @@ TEST(portability_analyser, positional_params_disabled)
   bpftrace->add_param("123");
   bpftrace->add_param("hello");
 
-  test(*bpftrace, "BEGIN { $1 }", 1);
-  test(*bpftrace, "BEGIN { str($2) }", 1);
+  test(*bpftrace, "begin { $1 }", 1);
+  test(*bpftrace, "begin { str($2) }", 1);
 }
 
 TEST(portability_analyser, curtask_disabled)
 {
-  test("BEGIN { curtask }", 1);
-  test("struct task_struct { char comm[16]; } BEGIN { curtask->comm }", 1);
+  test("begin { curtask }", 1);
+  test("struct task_struct { char comm[16]; } begin { curtask->comm }", 1);
 }
 
 TEST(portability_analyser, selective_probes_disabled)
 {
-  test("usdt:/bin/sh:probe { 1 }", 1);
-  test("usdt:/bin/sh:namespace:probe { 1 }", 1);
+  test("usdt:/bin/sh:tp1 { 1 }", 1);
+  test("usdt:/bin/sh:prov1:tp1 { 1 }", 1);
 
   auto bpftrace = get_mock_bpftrace();
   test(*bpftrace, "watchpoint:0x10000000:8:rw { 1 }", 1);
