@@ -1,20 +1,23 @@
-
+#include "ast/attachpoint_parser.h"
+#include "ast/passes/clang_build.h"
 #include "ast/passes/codegen_llvm.h"
 #include "ast/passes/field_analyser.h"
+#include "ast/passes/map_sugar.h"
+#include "ast/passes/named_param.h"
+#include "ast/passes/probe_expansion.h"
+#include "ast/passes/resolve_imports.h"
 #include "ast/passes/resource_analyser.h"
 #include "ast/passes/semantic_analyser.h"
-
+#include "ast/passes/type_system.h"
 #include "bpftrace.h"
-#include "clang_parser.h"
+#include "btf.h"
+#include "btf_common.h"
 #include "driver.h"
 #include "mocks.h"
-#include "gmock/gmock.h"
 #include "gtest/gtest.h"
 
 namespace bpftrace::test::probe {
 
-#include "btf_common.h"
-
 using bpftrace::ast::AttachPoint;
 using bpftrace::ast::AttachPointList;
 using bpftrace::ast::Probe;
@@ -22,29 +25,33 @@ using bpftrace::ast::Probe;
 void gen_bytecode(const std::string &input, std::stringstream &out)
 {
   auto bpftrace = get_mock_bpftrace();
-  Driver driver(*bpftrace);
+  ast::ASTContext ast("stdin", input);
 
-  ASSERT_EQ(driver.parse_str(input), 0);
+  ast::CDefinitions no_c_defs; // Output from clang parser.
 
-  ast::FieldAnalyser fields(driver.ctx, *bpftrace);
-  EXPECT_EQ(fields.analyse(), 0);
-
-  ClangParser clang;
-  clang.parse(driver.ctx.root, *bpftrace);
-
-  ast::SemanticAnalyser semantics(driver.ctx, *bpftrace);
-  ASSERT_EQ(semantics.analyse(), 0);
-
-  ast::ResourceAnalyser resource_analyser(driver.ctx, *bpftrace);
-  auto resources_optional = resource_analyser.analyse();
-  ASSERT_TRUE(resources_optional.has_value());
-  // clang-tidy doesn't recognize ASSERT_*() as execution terminating
-  // NOLINTNEXTLINE(bugprone-unchecked-optional-access)
-  bpftrace->resources = resources_optional.value();
-
-  ast::CodegenLLVM codegen(driver.ctx, *bpftrace);
-  codegen.generate_ir();
-  codegen.DumpIR(out);
+  // N.B. No macro expansion.
+  auto ok = ast::PassManager()
+                .put(ast)
+                .put<BPFtrace>(*bpftrace)
+                .put(no_c_defs)
+                .add(CreateParsePass())
+                .add(ast::CreateResolveImportsPass())
+                .add(ast::CreateParseAttachpointsPass())
+                .add(ast::CreateProbeExpansionPass())
+                .add(CreateParseBTFPass())
+                .add(ast::CreateMapSugarPass())
+                .add(ast::CreateFieldAnalyserPass())
+                .add(ast::CreateNamedParamsPass())
+                .add(ast::CreateLLVMInitPass())
+                .add(ast::CreateClangBuildPass())
+                .add(ast::CreateTypeSystemPass())
+                .add(ast::CreateSemanticPass())
+                .add(ast::CreateResourcePass())
+                .add(ast::AllCompilePasses())
+                .run();
+  ASSERT_TRUE(ok && ast.diagnostics().ok());
+  auto &obj = ok->get<ast::BpfObject>();
+  out.write(obj.data.data(), obj.data.size());
 }
 
 void compare_bytecode(const std::string &input1, const std::string &input2)
@@ -60,7 +67,8 @@ void compare_bytecode(const std::string &input1, const std::string &input2)
 
 TEST(probe, short_name)
 {
-  compare_bytecode("tracepoint:a:b { args }", "t:a:b { args }");
+  compare_bytecode("tracepoint:sched:sched_one { args }",
+                   "t:sched:sched_one { args }");
   compare_bytecode("kprobe:f { pid }", "k:f { pid }");
   compare_bytecode("kretprobe:f { pid }", "kr:f { pid }");
   compare_bytecode("uprobe:sh:f { 1 }", "u:sh:f { 1 }");
@@ -71,6 +79,14 @@ TEST(probe, short_name)
   compare_bytecode("interval:s:1 { 1 }", "i:s:1 { 1 }");
 }
 
+TEST(probe, case_insensitive)
+{
+  compare_bytecode("tracepoint:sched:sched_one { args }",
+                   "traCepoInt:sched:sched_one { args }");
+  compare_bytecode("kprobe:f { pid }", "KPROBE:f { pid }");
+  compare_bytecode("BEGIN { pid }", "begin { pid }");
+}
+
 class probe_btf : public test_btf {};
 
 TEST_F(probe_btf, short_name)
