@@ -11,8 +11,6 @@ pushd "$(dirname "${BASH_SOURCE[0]}")" >/dev/null 2>&1
 
 BPFTRACE_RUNTIME_TEST_EXECUTABLE=${BPFTRACE_RUNTIME_TEST_EXECUTABLE:-@CMAKE_BINARY_DIR@/src/bpftrace};
 export BPFTRACE_RUNTIME_TEST_EXECUTABLE;
-BPFTRACE_AOT_RUNTIME_TEST_EXECUTABLE=${BPFTRACE_AOT_RUNTIME_TEST_EXECUTABLE:-@CMAKE_BINARY_DIR@/src/aot/bpftrace-aotrt};
-export BPFTRACE_AOT_RUNTIME_TEST_EXECUTABLE;
 
 echo "===================="
 echo "bpftrace --info:"
