@@ -3,6 +3,7 @@
 from collections import namedtuple
 import os
 import platform
+from runner import Runner
 
 DEFAULT_TIMEOUT = 5
 
@@ -22,6 +23,7 @@ class Expect:
     self.expect = expect
     self.mode = mode
 
+# Remember to update tests/README.md if adding a new directive!
 TestStruct = namedtuple(
     'TestStruct',
     [
@@ -192,26 +194,7 @@ class TestParser(object):
             elif item_name == 'ARCH':
                 arch = [x.strip() for x in line.split("|")]
             elif item_name == 'REQUIRES_FEATURE':
-                features = {
-                    "loop",
-                    "btf",
-                    "fentry",
-                    "probe_read_kernel",
-                    "dpath",
-                    "uprobe_refcount",
-                    "signal",
-                    "iter",
-                    "libpath_resolv",
-                    "dwarf",
-                    "kernel_dwarf",
-                    "aot",
-                    "kprobe_multi",
-                    "uprobe_multi",
-                    "skboutput",
-                    "get_tai_ns",
-                    "get_func_ip",
-                    "jiffies64",
-                }
+                features = set(Runner.get_bpffeature().keys())
 
                 for f in line.split(" "):
                     f = f.strip()
@@ -233,7 +216,7 @@ class TestParser(object):
             elif item_name == "RETURN_CODE":
                 return_code = int(line.strip(' '))
             else:
-                raise UnknownFieldError('Field %s is unknown. Suite: %s' % (item_name, test_suite))
+                raise UnknownFieldError('Field %s is not a runtime test directive. Suite: %s' % (item_name, test_suite))
 
         if name == '':
             raise RequiredFieldError('Test NAME is required. Suite: ' + test_suite)
@@ -251,6 +234,9 @@ class TestParser(object):
         if return_code is None:
             return_code = 0
 
+        if return_code != 0 and will_fail:
+            raise InvalidFieldError('WILL_FAIL and RETURN_CODE can not be used together. Suite: ' + test_suite)
+
         return TestStruct(
             name,
             run,
