@@ -117,13 +117,13 @@ class Runner(object):
         nsenter_prefix = (" ".join(nsenter) + " ") if len(nsenter) > 0 else ""
 
         if test.run:
-            ret = re.sub("{{BPFTRACE}}", BPFTRACE_BIN, test.run)
+            ret = re.sub("{{BPFTRACE}}", BPFTRACE_BIN + " --verify-llvm-ir", test.run)
 
             return nsenter_prefix + ret
         else:  # PROG
             use_json = "-q -f json" if (len(test.expects) > 0 and test.expects[0].mode == "json") else ""
             escaped_prog = test.prog.replace("'", "'\\''")
-            cmd = nsenter_prefix + "{} {} -e '{}'".format(BPFTRACE_BIN, use_json, escaped_prog)
+            cmd = nsenter_prefix + "{} {} --verify-llvm-ir -e '{}'".format(BPFTRACE_BIN, use_json, escaped_prog)
             # We're only reusing PROG-directive tests for AOT tests
             if test.suite == 'aot':
                 return cmd + " --aot /tmp/tmpprog.btaot && /tmp/tmpprog.btaot"
@@ -136,7 +136,7 @@ class Runner(object):
 
     @staticmethod
     @lru_cache(maxsize=1)
-    def __get_bpffeature():
+    def get_bpffeature():
         p = subprocess.Popen(
             [BPFTRACE_BIN, "--info"],
             stdout=subprocess.PIPE,
@@ -151,19 +151,19 @@ class Runner(object):
         bpffeature["btf"] = output.find("btf: yes") != -1
         bpffeature["fentry"] = output.find("fentry: yes") != -1
         bpffeature["dpath"] = output.find("dpath: yes") != -1
-        bpffeature["uprobe_refcount"] = output.find("uprobe refcount") != -1
         bpffeature["signal"] = output.find("send_signal: yes") != -1
         bpffeature["iter"] = output.find("iter: yes") != -1
         bpffeature["libpath_resolv"] = output.find("bcc library path resolution: yes") != -1
-        bpffeature["dwarf"] = output.find("liblldb (DWARF support): yes") != -1
-        bpffeature["kernel_dwarf"] = output.find("Kernel DWARF: yes") != -1
+        bpffeature["dwarf"] = output.find("libdw (DWARF support): yes") != -1
         bpffeature["kprobe_multi"] = output.find("kprobe_multi: yes") != -1
+        bpffeature["kprobe_session"] = output.find("kprobe_session: yes") != -1
         bpffeature["uprobe_multi"] = output.find("uprobe_multi: yes") != -1
         bpffeature["aot"] = cmake_vars.LIBBCC_BPF_CONTAINS_RUNTIME
         bpffeature["skboutput"] = output.find("skboutput: yes") != -1
         bpffeature["get_tai_ns"] = output.find("get_ktime_ns: yes") != -1
         bpffeature["get_func_ip"] = output.find("get_func_ip: yes") != -1
         bpffeature["jiffies64"] = output.find("jiffies64: yes") != -1
+        bpffeature["lookup_percpu_elem"] = output.find("lookup_percpu_elem: yes") != -1
         return bpffeature
 
 
@@ -319,7 +319,7 @@ class Runner(object):
                             return Runner.SKIP_REQUIREMENT_UNSATISFIED
 
             if test.feature_requirement or test.neg_feature_requirement:
-                bpffeature = Runner.__get_bpffeature()
+                bpffeature = Runner.get_bpffeature()
 
                 for feature in test.feature_requirement:
                     if feature not in bpffeature:
@@ -394,8 +394,6 @@ class Runner(object):
                 if test.new_pidns:
                     # This can be fixed in the future if needed
                     raise ValueError(f"BEFORE_PID cannot be used with NEW_PIDNS")
-                if len(test.befores) > 1:
-                    raise ValueError(f"test has {len(test.befores)} BEFORE clauses but BEFORE_PID usage requires exactly one")
 
                 child_name = test.befores[0].strip().split()[-1]
                 child_name = os.path.basename(child_name)
@@ -406,7 +404,6 @@ class Runner(object):
                 'test': test.name,
                 '__BPFTRACE_NOTIFY_PROBES_ATTACHED': '1',
                 '__BPFTRACE_NOTIFY_AOT_PORTABILITY_DISABLED': '1',
-                'BPFTRACE_VERIFY_LLVM_IR': '1',
                 'PATH': os.environ.get('PATH', ''),
             }
             env.update(test.env)
@@ -520,13 +517,22 @@ class Runner(object):
             print(warn("[   SKIP   ] ") + label)
             return Runner.SKIP_AOT_NOT_SUPPORTED
 
-        if p and p.returncode != test.return_code and not test.will_fail and not timeout:
-            print(fail("[  FAILED  ] ") + label)
-            print('\tCommand: ' + bpf_call)
-            print('\tUnclean exit code: ' + str(p.returncode))
-            print('\tOutput: ' + to_utf8(output))
-            print_befores_and_after_output()
-            return Runner.FAIL
+        if p and not timeout:
+            if p.returncode != test.return_code and not test.will_fail:
+                print(fail("[  FAILED  ] ") + label)
+                print('\tCommand: ' + bpf_call)
+                print('\tUnclean exit code: ' + str(p.returncode))
+                print('\tOutput: ' + to_utf8(output))
+                print_befores_and_after_output()
+                return Runner.FAIL
+
+            if p.returncode == 0 and test.will_fail:
+                print(fail("[  FAILED  ] ") + label)
+                print("\tCommand: " + bpf_call)
+                print("\tClean exit code but expecting failure with WILL_FAIL directive")
+                print("\tOutput: " + to_utf8(output))
+                print_befores_and_after_output()
+                return Runner.FAIL
 
         if result:
             print(ok("[       OK ] ") + label)
