@@ -1,13 +1,11 @@
-#ifdef HAVE_SYSTEMTAP_SYS_SDT_H
-#include <sys/sdt.h>
-#else
-#define DTRACE_PROBE2(a, b, c, d) (void)0
-#endif
 #include "usdt_tp.h"
+
 #include <stdio.h>
 #include <sys/time.h>
 #include <unistd.h>
 
+#include "sdt.h"
+
 long myclock()
 {
   struct timeval tv;
