@@ -1,20 +1,17 @@
 #include <stdint.h>
 #include <stdio.h>
 
-struct T
-{
+struct T {
   uint32_t a;
   uint32_t b[2];
 };
 
-struct W
-{
+struct W {
   uint32_t a;
   struct T t;
 };
 
-struct C
-{
+struct C {
   uint32_t a;
   void* b;
   struct W w[10];
@@ -22,8 +19,7 @@ struct C
 
 void clear(struct C* c)
 {
-  for (int x = 0; x < 10; x++)
-  {
+  for (int x = 0; x < 10; x++) {
     c->w[x].t.a = 0;
   }
 }
@@ -34,8 +30,7 @@ int main()
 
   c.a = 0x55555555;
   c.b = (void*)0x55555555;
-  for (int x = 0; x < 10; x++)
-  {
+  for (int x = 0; x < 10; x++) {
     c.w[x].a = 100 + x;
     c.w[x].t.a = x;
     c.w[x].t.b[0] = 100 - x;
