@@ -9,8 +9,7 @@ int uprobeFunction1(int *n, char c __attribute__((unused)))
 
 void spin()
 {
-  while (1)
-  {
+  while (1) {
     int n = 13;
     char c = 'x';
     uprobeFunction1(&n, c);
@@ -21,8 +20,7 @@ void spin()
 int main(int argc __attribute__((unused)), char **argv __attribute__((unused)))
 {
   pid_t p = fork();
-  if (p < 0)
-  {
+  if (p < 0) {
     perror("fork fail");
     exit(1);
   }
