@@ -23,8 +23,8 @@ int uprobeArray(int (&array)[10])
 int main(int argc __attribute__((unused)), char **argv __attribute__((unused)))
 {
   int x = 42;
-  Foo foo{ 1, 2, 3, x };
-  Bar bar{ 10, 11, 12 };
+  Foo foo{ .a = 1, .b = 2, .c = 3, .x = x };
+  Bar bar{ .x = 10, .y = 11, .z = 12 };
   uprobeFunction1(x, foo, bar);
 
   int arr[10] = { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10 };
