@@ -1,12 +1,9 @@
-#ifdef HAVE_SYSTEMTAP_SYS_SDT_H
-#include <sys/sdt.h>
-#else
-#define DTRACE_PROBE2(a, b, c, d) (void)0
-#endif
 #include <stdio.h>
 #include <sys/time.h>
 #include <unistd.h>
 
+#include "sdt.h"
+
 __attribute__((always_inline)) inline static void myclock(int probe_num)
 {
   // Volatile forces reading directly from the stack so that
@@ -18,9 +15,13 @@ __attribute__((always_inline)) inline static void myclock(int probe_num)
   DTRACE_PROBE2(tracetest, testprobe, tv.tv_sec, on_stack);
 }
 
+__attribute__((always_inline)) inline static void mywrapper_inlined() {
+  myclock(100);
+}
+
 static void mywrapper()
 {
-  myclock(100);
+  mywrapper_inlined();
 }
 
 static void loop()
