@@ -1,14 +1,11 @@
-#define _SDT_HAS_SEMAPHORES 1
-
-#ifdef HAVE_SYSTEMTAP_SYS_SDT_H
-#include <sys/sdt.h>
-#else
-#define DTRACE_PROBE2(a, b, c, d) (void)0
-#endif
 #include <stdio.h>
 #include <sys/time.h>
 #include <unistd.h>
 
+#define _SDT_HAS_SEMAPHORES 1
+
+#include "sdt.h"
+
 __extension__ unsigned short tracetest_testprobe_semaphore
     __attribute__((unused)) __attribute__((section(".probes")))
     __attribute__((visibility("hidden")));
