@@ -1,6 +1,5 @@
 #include "types.h"
 #include "struct.h"
-#include "gmock/gmock.h"
 #include "gtest/gtest.h"
 
 namespace bpftrace::test::types {
@@ -23,8 +22,9 @@ TEST(types, to_str)
   EXPECT_EQ(to_str(CreateUInt32()), "uint32");
   EXPECT_EQ(to_str(CreateUInt64()), "uint64");
 
+  EXPECT_EQ(to_str(CreateString(10)), "string");
+
   EXPECT_EQ(to_str(CreateInet(10)), "inet[10]");
-  EXPECT_EQ(to_str(CreateString(10)), "string[10]");
   EXPECT_EQ(to_str(CreateBuffer(10)), "buffer[14]"); // metadata headroom
 
   EXPECT_EQ(to_str(CreatePointer(CreateInt8(), AddrSpace::kernel)), "int8 *");
@@ -33,16 +33,13 @@ TEST(types, to_str)
   ptr_ctx.MarkCtxAccess();
   EXPECT_EQ(to_str(ptr_ctx), "(ctx) int8 *");
 
-  EXPECT_EQ(to_str(CreateReference(CreateInt8(), AddrSpace::kernel)), "int8 &");
-
   EXPECT_EQ(to_str(CreateArray(2, CreateInt8())), "int8[2]");
 
-  auto record = std::weak_ptr<Struct>();
-  EXPECT_EQ(to_str(CreateRecord("hello", record)), "hello");
+  EXPECT_EQ(to_str(CreateRecord("hello")), "hello");
 
   std::shared_ptr<Struct> tuple = Struct::CreateTuple(
       { CreateInt8(), CreateString(10) });
-  EXPECT_EQ(to_str(CreateTuple(tuple)), "(int8,string[10])");
+  EXPECT_EQ(to_str(CreateTuple(std::move(tuple))), "(int8,string)");
 
   EXPECT_EQ(to_str(CreateSum(true)), "sum_t");
   EXPECT_EQ(to_str(CreateSum(false)), "usum_t");
@@ -59,8 +56,7 @@ TEST(types, to_str)
   EXPECT_EQ(to_str(CreateStats(true)), "stats_t");
   EXPECT_EQ(to_str(CreateStats(false)), "ustats_t");
 
-  EXPECT_EQ(to_str(CreateCount(true)), "count_t");
-  EXPECT_EQ(to_str(CreateCount(false)), "ucount_t");
+  EXPECT_EQ(to_str(CreateCount()), "count_t");
 
   EXPECT_EQ(to_str(CreateMacAddress()), "mac_address");
   EXPECT_EQ(to_str(CreateStack(true)), "kstack");
@@ -75,6 +71,7 @@ TEST(types, to_str)
   EXPECT_EQ(to_str(CreateStrerror()), "strerror_t");
   EXPECT_EQ(to_str(CreateHist()), "hist_t");
   EXPECT_EQ(to_str(CreateLhist()), "lhist_t");
+  EXPECT_EQ(to_str(CreateTSeries()), "tseries_t");
   EXPECT_EQ(to_str(CreateNone()), "none");
   EXPECT_EQ(to_str(CreateVoid()), "void");
 }
